@@ -404,7 +404,7 @@ namespace ArcDMCRucio {
 
 } // namespace ArcDMCRucio
 
-Arc::PluginDescriptor ARC_PLUGINS_TABLE_NAME[] = {
+extern Arc::PluginDescriptor const ARC_PLUGINS_TABLE_NAME[] = {
   { "rucio", "HED:DMC", "ATLAS Data Management System", 0, &ArcDMCRucio::DataPointRucio::Instance },
   { NULL, NULL, NULL, 0, NULL }
 };
