@@ -20,8 +20,8 @@
 
 #include <arc/Logger.h>
 #include <arc/Utils.h>
-//#include <arc/globusutils/GSSCredential.h>
-#include <arc/credential/Credential.h>
+
+#include "cert_util.h"
 
 #include "unixmap.h"
 
@@ -104,10 +104,13 @@ void recover_lcmaps_env(void) {
 }
 
 gss_cred_id_t read_globus_credentials(const std::string& filename) {
-  Arc::Credential cred(filename, "", "", "", "", true);
-  X509* cert = cred.GetCert();
-  STACK_OF(X509)* cchain = cred.GetCertChain();
-  EVP_PKEY* key = cred.GetPrivKey();
+  X509* cert = NULL;
+  STACK_OF(X509)* cchain = NULL;
+  EVP_PKEY* key = NULL;
+
+  LoadCertificateFile(filename, cert, cchain);
+  LoadKeyFile(filename, key);
+
   globus_gsi_cred_handle_t chandle;
   globus_gsi_cred_handle_init(&chandle, NULL);
   if(cert) globus_gsi_cred_set_cert(chandle, cert);
