@@ -61,15 +61,18 @@ namespace DataStaging {
        force_registration(false),
        status(DTRStatus::NEW,"Created by the generator"),
        bytes_transferred(0),
+       transfer_time(0),
        created(time(NULL)),
        cancel_request(false),
        bulk_start(false),
        bulk_end(false),
        source_supports_bulk(false),
+       mandatory(true),
        delivery_endpoint(LOCAL_DELIVERY),
        use_host_cert_for_remote_delivery(false),
        current_owner(GENERATOR),
-       logger(log)
+       logger(log),
+       perf_record(perf_log)
   {
     if (!logger) {
       // use root logger if none is supplied
@@ -119,6 +122,9 @@ namespace DataStaging {
 #else
     cache_state = (source_endpoint->Cache() && destination_endpoint->Local()) ? CACHEABLE : NON_CACHEABLE;
 #endif
+    if (source_url.Option("failureallowed") == "yes" || destination_url.Option("failureallowed") == "yes") {
+      mandatory = false;
+    }
     
     /* Think how to populate transfer parameters */
     mark_modification();
@@ -158,6 +164,7 @@ namespace DataStaging {
     cache_file.clear();
     mapped_source.clear();
     bytes_transferred = 0;
+    transfer_time = 0;
     reset_error_status();
   }
 
@@ -244,6 +251,10 @@ namespace DataStaging {
     bytes_transferred = bytes;
   }
 
+  void DTR::set_transfer_time(unsigned long long int t) {
+    transfer_time = t;
+  }
+
   void DTR::set_cache_file(const std::string& filename)
   {
     cache_file = filename;
