@@ -43,6 +43,7 @@ namespace DataStaging {
 #include <arc/CheckSum.h>
 #include <arc/data/URLMap.h>
 #include <arc/DateTime.h>
+#include <arc/JobPerfLog.h>
 #include <arc/Logger.h>
 #include <arc/User.h>
 #include <arc/UserConfig.h>
@@ -285,6 +286,8 @@ namespace DataStaging {
 
     /// Number of bytes transferred so far
     unsigned long long int bytes_transferred; // TODO and/or offset?
+    /// Time taken in ns to complete transfer (0 if incomplete)
+    unsigned long long int transfer_time;
 
     /** Timing variables **/
     /// When should we finish the current action
@@ -306,6 +309,9 @@ namespace DataStaging {
     /// Whether bulk operations are supported for the source
     bool source_supports_bulk;
 
+    /// Flag to say whether success of the DTR is mandatory
+    bool mandatory;
+
     /// Endpoint of delivery service this DTR is scheduled for.
     /** By default it is LOCAL_DELIVERY so local Delivery is used. */
     Arc::URL delivery_endpoint;
@@ -330,6 +336,12 @@ namespace DataStaging {
      * to avoid duplicate messages */
     std::list<Arc::LogDestination*> log_destinations;
 
+    /// Performance metric logger
+    Arc::JobPerfLog perf_log;
+
+    /// Performance record used for recording transfer time
+    Arc::JobPerfRecord perf_record;
+
     /// List of callback methods called when DTR moves between processes
     std::map<StagingProcesses,std::list<DTRCallback*> > proc_callback;
 
@@ -504,6 +516,11 @@ namespace DataStaging {
     /// Get current number of bytes transferred
     unsigned long long int get_bytes_transferred() const { return bytes_transferred; };
 
+    /// Set transfer time (should be set by whatever is controlling the transfer)
+    void set_transfer_time(unsigned long long int t);
+    /// Get transfer time
+    unsigned long long int get_transfer_time() const { return transfer_time; };
+
     /// Set the DTR to be cancelled
     void set_cancel_request();
     /// Returns true if cancellation has been requested
@@ -579,6 +596,9 @@ namespace DataStaging {
     /// Whether bulk operation is possible according to current state and src/dest
     bool bulk_possible();
 
+    /// Whether DTR success is mandatory
+    bool is_mandatory() const { return mandatory; };
+
     /// Get Logger object, so that processes can log to this DTR's log
     const DTRLogger& get_logger() const { return logger; };
 
@@ -613,6 +633,12 @@ namespace DataStaging {
     bool came_from_generator() const;
     /// Returns true if this DTR is in a final state (finished, failed or cancelled)
     bool is_in_final_state() const;
+
+    /// Get the performance log
+    Arc::JobPerfLog& get_job_perf_log() { return perf_log; };
+    /// Get the performance log record
+    Arc::JobPerfRecord& get_job_perf_record() { return perf_record; };
+
   };
   
   /// Helper method to create smart pointer, only for swig bindings
