@@ -222,10 +222,11 @@ namespace DataStaging {
     Glib::Mutex::Lock lock(lock_);
     if (!client) return;
 
-    // check time since last query - for long transfers we do not need to query
-    // at a high frequency. After 5s query every 5s.
+    // check time since last query - check every second for the first 20s and
+    // after every 5s
     // TODO be more intelligent, using transfer rate and file size
-    if (Arc::Time() - start_ > 5 && Arc::Time() - Arc::Time(status_.timestamp) < 5) return;
+    if (Arc::Time() - start_ < 20 && Arc::Time() - Arc::Time(status_.timestamp) < 1) return;
+    if (Arc::Time() - start_ > 20 && Arc::Time() - Arc::Time(status_.timestamp) < 5) return;
 
     Arc::NS ns;
     Arc::PayloadSOAP request(ns);
@@ -298,7 +299,7 @@ namespace DataStaging {
     delete response;
   }
 
-  bool DataDeliveryRemoteComm::CheckComm(DTR_ptr dtr, std::vector<std::string>& allowed_dirs) {
+  bool DataDeliveryRemoteComm::CheckComm(DTR_ptr dtr, std::vector<std::string>& allowed_dirs, std::string& load_avg) {
     // call Ping
     Arc::MCCConfig cfg;
     if (dtr->host_cert_for_remote_delivery()) {
@@ -378,6 +379,12 @@ namespace DataStaging {
                              (std::string)dir, dtr->get_delivery_endpoint().str());
     }
 
+    if (resultnode["LoadAvg"]) {
+      load_avg = (std::string)(resultnode["LoadAvg"]);
+    } else {
+      load_avg = "-1";
+    }
+
     delete response;
     return true;
   }
@@ -396,6 +403,7 @@ namespace DataStaging {
       status_.streams = 0;
       status_.transferred = 0;
       status_.size = 0;
+      status_.transfer_time = 0;
       status_.offset = 0;
       status_.speed = 0;
       strncpy(status_.checksum, empty.c_str(), sizeof(status_.checksum));
@@ -444,6 +452,12 @@ namespace DataStaging {
       Arc::stringto(std::string(datanode), bytes);
       status_.transferred = bytes;
     }
+    datanode = node["TransferTime"];
+    if (datanode) {
+      unsigned long long int t;
+      Arc::stringto(std::string(datanode), t);
+      status_.transfer_time = t;
+    }
     // TODO size, offset, speed (currently not used)
     datanode = node["CheckSum"];
     if (datanode) {
