@@ -17,6 +17,7 @@
 #include <arc/message/PayloadStream.h>
 #include <arc/message/SecAttr.h>
 #include <arc/ws-addressing/WSA.h>
+#include <arc/JobPerfLog.h>
 #include <arc/Thread.h>
 #include <arc/StringConv.h>
 #include <arc/FileUtils.h>
@@ -441,6 +442,7 @@ ARexConfigContext* ARexService::get_configuration(Arc::Message& inmsg) {
   if(mcontext) {
     try {
       config = dynamic_cast<ARexConfigContext*>(mcontext);
+      logger_.msg(Arc::DEBUG,"Using cached local account '%s'", config->User().Name());
     } catch(std::exception& e) { };
   };
   if(config) return config;
@@ -731,13 +733,41 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         inpayload->Namespaces(ns_);
         CountedResourceLock cl_lock(beslimit_);
         std::string credentials;
-        if(!delegation_stores_.Process(config->GmConfig().DelegationDir(),*inpayload,*outpayload,config->GridName(),credentials)) {
+        if(!delegation_stores_.Process(config->GmConfig().DelegationDir(),
+                                *inpayload,*outpayload,config->GridName(),credentials)) {
           delete outpayload;
           return make_soap_fault(outmsg);
         };
-        if((!credentials.empty()) && (MatchXMLNamespace(op,DELEG_ARC_NAMESPACE))) {
-          // Only ARC delegation is done per job
-          UpdateCredentials(*config,op,outpayload->Child(),credentials);
+        if(!credentials.empty()) {
+          // Credentials obtained as outcome of operation
+          if(MatchXMLNamespace(op,DELEG_ARC_NAMESPACE)) {
+            // ARC delegation is done per job but stored under
+            // own id. So storing must be done outside processing code.
+            UpdateCredentials(*config,op,outpayload->Child(),credentials);
+          } else if(MatchXMLNamespace(op,ES_DELEG_NAMESPACE)) {
+            // ES has delegations assigned their own ids and are
+            // already updated in delegation_stores_.Process()
+#if 1
+            // But for compatibility during intermediate period store delegations in
+            // per-job proxy file too.
+            // Obtaining all jobs associated with that delegation id would cause 
+            // scanning all job.#.local files for associates delegationid.
+            // To avoid this costly procedure job.#.proxy will be updated 
+            // when job state changes (including restart).
+            /*
+            if(op.Name() == "PutDelegation") {
+              // PutDelegation
+              //   DelegationID
+              //   Credential
+              std::string id = op["DelegationId"];
+              if(!id.empty()) {
+                delegation_stores_[config->GmConfig().DelegationDir()].
+
+              };
+            };
+            */
+#endif
+          };
         };
       } else if(config_.ARCInterfaceEnabled() && MatchXMLNamespace(op,WSRF_NAMESPACE)) {
         CountedResourceLock cl_lock(infolimit_);
@@ -885,6 +915,7 @@ void ARexService::gm_threads_starter() {
 
 ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Service(cfg,parg),
               logger_(Arc::Logger::rootLogger, "A-REX"),
+              delegation_stores_(),
               infodoc_(true),
               inforeg_(NULL),
               infoprovider_wakeup_period_(0),
@@ -892,6 +923,7 @@ ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Servic
               gm_(NULL) {
   valid = false;
   config_.SetJobLog(new JobLog());
+  config_.SetJobPerfLog(new Arc::JobPerfLog());
   config_.SetContPlugins(new ContinuationPlugins());
   config_.SetCredPlugin(new RunPlugin());
   // logger_.addDestination(logcerr);
@@ -971,6 +1003,20 @@ ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Servic
     return;
   }
 
+  // Pass information about delegation db type
+  {
+  DelegationStore::DbType deleg_db_type = DelegationStore::DbBerkeley;
+    switch(config_.DelegationDBType()) {
+     case GMConfig::deleg_db_bdb:
+      deleg_db_type = DelegationStore::DbBerkeley;
+      break;
+     case GMConfig::deleg_db_sqlite:
+      deleg_db_type = DelegationStore::DbSQLite;
+      break;
+    };
+    delegation_stores_.SetDbType(deleg_db_type);
+  };
+
   // Set default queue if none given
   if(config_.DefaultQueue().empty() && (config_.Queues().size() == 1)) {
     config_.SetDefaultQueue(config_.Queues().front());
@@ -1051,7 +1097,7 @@ ARexService::~ARexService(void) {
 
 } // namespace ARex
 
-Arc::PluginDescriptor ARC_PLUGINS_TABLE_NAME[] = {
+extern Arc::PluginDescriptor const ARC_PLUGINS_TABLE_NAME[] = {
     { "a-rex", "HED:SERVICE", NULL, 0, &ARex::get_service },
     { NULL, NULL, NULL, 0, NULL }
 };
