@@ -12,6 +12,13 @@
 
 #include <arc/FileUtils.h>
 
+#define DELEGATION_USES_SQLITE 1
+
+#ifdef HAVE_SQLITE
+#include "FileRecordSQLite.h"
+#endif
+#include "FileRecordBDB.h"
+
 #include "DelegationStore.h"
 
 namespace ARex {
@@ -23,17 +30,31 @@ namespace ARex {
     Arc::DirCreate(dpath,0,0,S_IXUSR|S_IRUSR|S_IWUSR,true);
   }
 
-  DelegationStore::DelegationStore(const std::string& base, bool allow_recover):
+  DelegationStore::DelegationStore(const std::string& base, DbType db, bool allow_recover):
            logger_(Arc::Logger::rootLogger, "Delegation Storage") {
+    fstore_ = NULL;
     expiration_ = 0;
     maxrecords_ = 0;
     mtimeout_ = 0;
     mrec_ = NULL;
-    fstore_ = new FileRecord(base, allow_recover);
+    switch(db) {
+      case DbBerkeley:
+        fstore_ = new FileRecordBDB(base, allow_recover);
+        break;
+#ifdef HAVE_SQLITE
+      case DbSQLite:
+        fstore_ = new FileRecordSQLite(base, allow_recover);
+        break;
+#endif
+      default:
+        failure_ = "Unsupported database type requested for delegation storage.";
+        logger_.msg(Arc::ERROR,"%s",failure_);
+        return;
+    };
     if(!*fstore_) {
       failure_ = "Failed to initialize storage. " + fstore_->Error();
+      logger_.msg(Arc::WARNING,"%s",failure_);
       if(allow_recover) {
-        logger_.msg(Arc::WARNING,"%s",failure_);
         // Database creation failed. Try recovery.
         if(!fstore_->Recover()) {
           failure_ = "Failed to recover storage. " + fstore_->Error();
@@ -55,7 +76,19 @@ namespace ARex {
               };
             };
           };
-          fstore_ = new FileRecord(base);
+          switch(db) {
+            case DbBerkeley:
+              fstore_ = new FileRecordBDB(base);
+              break;
+#ifdef HAVE_SQLITE
+            case DbSQLite:
+              fstore_ = new FileRecordSQLite(base);
+              break;
+#endif
+            default:
+              // Must not happen - already sorted out above.
+              return;
+          };
           if(!*fstore_) {
             // Failure
             failure_ = "Failed to re-create storage. " + fstore_->Error();
@@ -239,22 +272,69 @@ namespace ARex {
   void DelegationStore::PeriodicCheckConsumers(void) {
     // Go through stored credentials
     // Remove outdated records (those with locks won't be removed)
-    time_t start = ::time(NULL);
     if(expiration_) {
+      time_t start = ::time(NULL);
       Glib::Mutex::Lock check_lock(lock_);
-      if(mrec_ == NULL) mrec_ = new FileRecord::Iterator(*fstore_);
+      if(mrec_ != NULL) {
+        if(!mrec_->resume()) {
+          logger_.msg(Arc::WARNING,"DelegationStore: PeriodicCheckConsumers failed to resume iterator");
+          delete mrec_;
+          mrec_ = NULL;
+        };
+      };
+      if(mrec_ == NULL) {
+        mrec_ = fstore_->NewIterator();
+      };
       for(;(bool)(*mrec_);++(*mrec_)) {
-        if(mtimeout_ && (((unsigned int)(::time(NULL) - start)) > mtimeout_)) return;
+        if(mtimeout_ && (((unsigned int)(::time(NULL) - start)) > mtimeout_)) {
+          mrec_->suspend();
+          return;
+        }
         struct stat st;
         if(::stat(mrec_->path().c_str(),&st) == 0) {
           if(((unsigned int)(::time(NULL) - st.st_mtime)) > expiration_) {
-            fstore_->Remove(mrec_->id(),mrec_->owner());
+            if(fstore_->Remove(mrec_->id(),mrec_->owner())) {
+            } else {
+              // It is ok to fail here because Remove checks for delegation locks.
+              // So reporting only for debuging purposes.
+              logger_.msg(Arc::DEBUG,"DelegationStore: PeriodicCheckConsumers failed to remove old delegation %s - %s", mrec_->uid(), fstore_->Error());
+            };
           };    
         };
       };
       delete mrec_; mrec_ = NULL;
     };
     // TODO: Remove records over threshold
+    return;
+  }
+
+  bool DelegationStore::AddCred(std::string& id, const std::string& client, const std::string& credentials) {
+    std::string path = fstore_->Add(id,client,std::list<std::string>());
+    if(path.empty()) {
+      failure_ = "Local error - failed to create slot for delegation. "+fstore_->Error();
+      return false;
+    }
+    make_dir_for_file(path);
+    if(!Arc::FileCreate(path,credentials,0,0,S_IRUSR|S_IWUSR)) {
+      fstore_->Remove(id,client);
+      failure_ = "Local error - failed to create storage for delegation";
+      return false;
+    };
+    return true;
+  }
+
+  bool DelegationStore::PutCred(const std::string& id, const std::string& client, const std::string& credentials) {
+    std::list<std::string> meta;
+    std::string path = fstore_->Find(id,client,meta);
+    if(path.empty()) {
+      failure_ = "Local error - failed to find specified credentials. "+fstore_->Error();
+      return false;
+    }
+    if(!Arc::FileCreate(path,credentials,0,0,S_IRUSR|S_IWUSR)) {
+      failure_ = "Local error - failed to store delegation";
+      return false;
+    };
+    return true;
   }
 
   std::string DelegationStore::FindCred(const std::string& id,const std::string& client) {
@@ -262,15 +342,45 @@ namespace ARex {
     return fstore_->Find(id,client,meta);
   }
 
+  bool DelegationStore::GetCred(const std::string& id, const std::string& client, std::string& credentials) {
+    std::list<std::string> meta;
+    std::string path = fstore_->Find(id,client,meta);
+    if(path.empty()) {
+      failure_ = "Local error - failed to find specified credentials. "+fstore_->Error();
+      return false;
+    }
+    std::string content;
+    if(!Arc::FileRead(path,credentials)) {
+      failure_ = "Local error - failed to read credentials";
+      return false;
+    };
+    return true;
+  }
+
+  bool DelegationStore::GetLocks(const std::string& id, const std::string& client, std::list<std::string>& lock_ids) {
+    return fstore_->ListLocks(id, client, lock_ids);
+  }
+
+  bool DelegationStore::GetLocks(std::list<std::string>& lock_ids) {
+    return fstore_->ListLocks(lock_ids);
+  }
+
   std::list<std::string> DelegationStore::ListCredIDs(const std::string& client) {
     std::list<std::string> res;
-    FileRecord::Iterator rec(*fstore_);
+    FileRecord::Iterator& rec = *(fstore_->NewIterator());
     for(;(bool)rec;++rec) {
       if(rec.owner() == client) res.push_back(rec.id());
     };
+    delete &rec;
     return res;
   }
 
+  std::list<std::pair<std::string,std::string> > DelegationStore::ListLockedCredIDs(const std::string& lock_id) {
+    std::list<std::pair<std::string,std::string> > ids;
+    (void)fstore_->ListLocked(lock_id, ids);
+    return ids;
+  }
+
   std::list<std::string> DelegationStore::ListLockedCredIDs(const std::string& lock_id, const std::string& client) {
     std::list<std::string> res;
     std::list<std::pair<std::string,std::string> > ids;
@@ -284,16 +394,21 @@ namespace ARex {
 
   std::list<std::pair<std::string,std::string> > DelegationStore::ListCredIDs(void) {
     std::list<std::pair<std::string,std::string> > res;
-    FileRecord::Iterator rec(*fstore_);
+    FileRecord::Iterator& rec = *(fstore_->NewIterator());
     for(;(bool)rec;++rec) {
       res.push_back(std::pair<std::string,std::string>(rec.id(),rec.owner()));
     };
+    delete &rec;
     return res;
   }
 
 
   bool DelegationStore::LockCred(const std::string& lock_id, const std::list<std::string>& ids,const std::string& client) {
-    return fstore_->AddLock(lock_id,ids,client);
+    if(!fstore_->AddLock(lock_id,ids,client)) {
+      failure_ = "Local error - failed set lock for delegation. "+fstore_->Error();
+      return false;
+    };
+    return true;
   }
 
   bool DelegationStore::ReleaseCred(const std::string& lock_id, bool touch, bool remove) {
