@@ -7,6 +7,7 @@
 
 #include <arc/ArcConfig.h>
 #include <arc/ArcLocation.h>
+#include <arc/JobPerfLog.h>
 #include <arc/StringConv.h>
 #include <arc/Utils.h>
 #include <arc/XMLNode.h>
@@ -125,11 +126,16 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
 
   // List of helper commands that will be substituted after all configuration is read
   std::list<std::string> helpers;
+  bool helper_log_is_set = false;
   ConfigSections cf(cfile);
   cf.AddSection("common");
   cf.AddSection("grid-manager");
   cf.AddSection("infosys");
   cf.AddSection("queue");
+  if (config.job_perf_log) {
+    config.job_perf_log->SetEnabled(false);
+    config.job_perf_log->SetOutput("/var/log/arc/perfdata/data.perflog");
+  }
   // process configuration information here
   for(;;) {
     std::string rest;
@@ -141,7 +147,7 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
       }
       continue;
     }
-    if (cf.SectionNum() == 0) { // infosys user may be in common too
+    if (cf.SectionNum() == 0) { // common - infosys user may be in common too
       if (command == "user") {
         config.SetShareID(Arc::User(rest));
       } else if(command == "x509_cert_dir") {
@@ -175,6 +181,18 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
       std::string fname = config_next_arg(rest);  // empty is allowed too
       config.job_log->SetOutput(fname.c_str());
     }
+    else if (command == "enable_perflog_reporting") { //
+      if (!config.job_perf_log) continue;
+      bool enable = false;
+      if (!CheckYesNoCommand(enable, command, rest)) return false;
+      config.job_perf_log->SetEnabled(enable);
+    }
+    else if (command == "perflogdir") { // 
+      if (!config.job_perf_log) continue;
+      std::string fname = config_next_arg(rest);  // empty is allowed too
+      if(!fname.empty()) fname += "/arex.perflog";
+      config.job_perf_log->SetOutput(fname.c_str());
+    }
     else if (command == "jobreport") { // service to report information to
       if (!config.job_log) continue;
       for(;;) {
@@ -358,6 +376,18 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
         logger.msg(Arc::ERROR, "Wrong option in fixdirectories"); return false;
       }
     }
+    else if (command == "delegationdb") {
+      std::string s = config_next_arg(rest);
+      if (s == "bdb") {
+        config.deleg_db = GMConfig::deleg_db_bdb;
+      }
+      else if (s == "sqlite") {
+        config.deleg_db = GMConfig::deleg_db_sqlite;
+      }
+      else {
+        logger.msg(Arc::ERROR, "Wrong option in delegationdb"); return false;
+      }
+    }
     else if (command == "allowsubmit") { // Note: not available in xml
       config.allow_submit += " " + config_next_arg(rest);
     }
@@ -405,9 +435,18 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
       }
       helpers.push_back(rest);
     }
+    else if (command == "helperlog") {
+      config.helper_log = config_next_arg(rest);
+      // empty is allowed
+      helper_log_is_set = true;
+    }
   }
   // End of parsing conf commands
 
+  if(!helper_log_is_set) {
+    // Assgn default backward compatible value
+    config.helper_log = config.control_dir + "/job.helper.errors";
+  }
   // Do substitution of control dir and helpers here now we have all the
   // configuration. These are special because they do not change per-user
   config.Substitute(config.control_dir);
@@ -444,6 +483,7 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
   // So we do not need any special treatment for infosys.
   // std::string infosys_user("");
 
+  bool helper_log_is_set = false;
   Arc::XMLNode tmp_node = cfg["endpoint"];
   if (tmp_node) config.headnode = (std::string)tmp_node;
   /*
@@ -679,6 +719,10 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
   int n;
   if (!elementtoenum(tmp_node, "fixDirectories", n=(int)fixdir, fixdir_opts, &logger)) return false;
   config.fixdir = (GMConfig::fixdir_t)n;
+  GMConfig::deleg_db_t deleg_db = GMConfig::deleg_db_bdb;
+  const char* deleg_db_opts[] = { "bdb", "sqlite", NULL };
+  if (!elementtoenum(tmp_node, "delegationDB", n=(int)deleg_db, deleg_db_opts, &logger)) return false;
+  config.deleg_db = (GMConfig::deleg_db_t)n;
   if (!elementtoint(tmp_node, "maxReruns", config.reruns, &logger)) return false;
   if (!elementtobool(tmp_node, "noRootPower", config.strict_session, &logger)) return false;
   if (!elementtoint(tmp_node, "defaultTTL", config.keep_finished, &logger)) return false;
@@ -694,6 +738,15 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
     return false;
   }
 
+  /*
+  helperLog
+  */
+  tmp_node = cfg["helperLog"];
+  if(tmp_node) {
+    config.helper_log = (std::string)tmp_node;
+    helper_log_is_set = true;
+  };
+
   /*
   helperUtility
     username
@@ -720,6 +773,10 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
   }
   // End of parsing XML node
 
+  if(!helper_log_is_set) {
+    // Set default
+    config.helper_log = config.control_dir + "/job.helper.errors";
+  }
   // Do substitution of control dir and helpers here now we have all the
   // configuration. These are special because they do not change per-user
   config.Substitute(config.control_dir);
