@@ -67,6 +67,7 @@ GMConfig::GMConfig(const Arc::XMLNode& node): xml_cfg(node) {
 void GMConfig::SetDefaults() {
   conffile_is_temp = false;
   job_log = NULL;
+  job_perf_log = NULL;
   cont_plugins = NULL;
   cred_plugin = NULL;
   delegations = NULL;
@@ -84,6 +85,8 @@ void GMConfig::SetDefaults() {
   max_jobs = -1;
   max_jobs_per_dn = -1;
 
+  deleg_db = deleg_db_bdb;
+
   enable_arc_interface = true;
   enable_emies_interface = false;
 
@@ -265,6 +268,10 @@ std::string GMConfig::DelegationDir() const {
   return deleg_dir;
 }
 
+GMConfig::deleg_db_t GMConfig::DelegationDBType() const {
+  return deleg_db;
+}
+
 bool GMConfig::Substitute(std::string& param, const Arc::User& user) const {
   std::string::size_type curpos = 0;
   for (;;) {
@@ -364,6 +371,22 @@ GMConfig::ExternalHelper::~ExternalHelper() {
   }
 }
 
+static void ExternalHelperInitializer(void* arg) {
+  const char* logpath = reinterpret_cast<const char*>(arg);
+  // set up stdin,stdout and stderr
+  int h;
+  h = ::open("/dev/null",O_RDONLY);
+  if(h != 0) { if(dup2(h,0) != 0) { sleep(10); _exit(1); }; close(h); };
+  h = ::open("/dev/null",O_WRONLY);
+  if(h != 1) { if(dup2(h,1) != 1) { sleep(10); _exit(1); }; close(h); };
+  if(logpath && logpath[0]) {
+    h = ::open(logpath,O_WRONLY | O_CREAT | O_APPEND,S_IRUSR | S_IWUSR);
+    if(h==-1) { h = ::open("/dev/null",O_WRONLY); };
+  }
+  else { h = ::open("/dev/null",O_WRONLY); };
+  if(h != 2) { if(dup2(h,2) != 2) { sleep(10); exit(1); }; close(h); };
+}
+
 bool GMConfig::ExternalHelper::run(const GMConfig& config) {
   if (proc != NULL) {
     if (proc->Running()) {
@@ -375,11 +398,14 @@ bool GMConfig::ExternalHelper::run(const GMConfig& config) {
   // start/restart
   if (command.empty()) return true;  // has anything to run ?
   logger.msg(Arc::VERBOSE, "Starting helper process: %s", command);
-  std::string helper_id = "helper";
-  bool started = RunParallel::run(config, Arc::User(), helper_id.c_str(), command, &proc);
-  if (started) return true;
-  if (proc && (*proc)) return true;
-  if (proc) { delete proc; proc = NULL; };
+
+  proc = new Arc::Run(command);
+  proc->KeepStdin(true);
+  proc->KeepStdout(true);
+  proc->KeepStderr(true);
+  proc->AssignInitializer(&ExternalHelperInitializer, const_cast<char*>(config.HelperLog().c_str()));
+  if (proc->Start()) return true;
+  delete proc;
   logger.msg(Arc::ERROR, "Helper process start failed: %s", command);
   // start failed, doing nothing - maybe in the future
   return false;
