@@ -3,6 +3,7 @@
 
 #include <string>
 
+#include <arc/JobPerfLog.h>
 #include <arc/Logger.h>
 #include <arc/Run.h>
 #include <arc/User.h>
@@ -54,6 +55,11 @@ public:
    fixdir_never
   };
 
+  enum deleg_db_t {
+    deleg_db_bdb,
+    deleg_db_sqlite
+  };
+
   /// Use given (or guessed if not given) configuration file.
   /**
    * Guessing uses $ARC_CONFIG, $ARC_LOCATION/etc/arc.conf or the default
@@ -119,12 +125,18 @@ public:
   const std::string& RTEDir() const { return rte_dir; }
   /// Directory storing delegations
   std::string DelegationDir() const;
+  /// Database type to use for delegation storage
+  deleg_db_t DelegationDBType() const;
+  /// Helper(s) log file path
+  const std::string& HelperLog() const { return helper_log; }
 
   /// email address of person responsible for this ARC installation
   const std::string& SupportMailAddress() const { return support_email_address; }
 
   /// Set JobLog object
   void SetJobLog(JobLog* log) { job_log = log; }
+  /// Set JobPerfLog object
+  void SetJobPerfLog(Arc::JobPerfLog* log) { job_perf_log = log; }
   /// Set ContinuationPlugins (plugins run at state transitions)
   void SetContPlugins(ContinuationPlugins* plugins) { cont_plugins = plugins; }
   /// Set RunPlugin (plugin used to acquire local credentials)
@@ -133,6 +145,8 @@ public:
   void SetDelegations(ARex::DelegationStores* stores) { delegations = stores; }
   /// JobLog object
   JobLog* GetJobLog() const { return job_log; }
+  /// JobPerfLog object
+  Arc::JobPerfLog* GetJobPerfLog() const { return job_perf_log; }
   /// Plugins run at state transitions
   ContinuationPlugins* ContPlugins() const { return cont_plugins; }
   /// Plugin used to acquire local credentials
@@ -234,6 +248,8 @@ private:
   Arc::XMLNode xml_cfg;
   /// For logging job information to external logging service
   JobLog* job_log;
+  /// For logging performace/profiling information
+  Arc::JobPerfLog* job_perf_log;
   /// Plugins run at certain state changes
   ContinuationPlugins* cont_plugins;
   /// Plugin for acquiring local credentials
@@ -250,6 +266,8 @@ private:
   std::string rte_dir;
   /// email address for support
   std::string support_email_address;
+  /// helper(s) log path
+  std::string helper_log;
 
   /// Scratch directory
   std::string scratch_dir;
@@ -306,6 +324,8 @@ private:
   std::string gridftp_endpoint;
   /// WS-interface endpoint
   std::string arex_endpoint;
+  /// Delegation db type
+  deleg_db_t deleg_db;
 
   /// Logger object
   static Arc::Logger logger;
