@@ -28,6 +28,7 @@ StagingConfig::StagingConfig(const GMConfig& config):
   log_level(Arc::Logger::getRootLogger().getThreshold()),
   valid(true)
 {
+  perf_log.SetOutput("/var/log/arc/perfdata/data.perflog");
 
   // For ini-style, use [data-staging] section, for xml use <dataTransfer> node
 
@@ -70,6 +71,7 @@ bool StagingConfig::readStagingConf(std::ifstream& cfile) {
 
   ConfigSections cf(cfile);
   cf.AddSection("data-staging");
+  cf.AddSection("common");
   for(;;) {
     std::string rest;
     std::string command;
@@ -189,6 +191,13 @@ bool StagingConfig::readStagingConf(std::ifstream& cfile) {
       endpoint.replace(0, endpoint.find("://"), "acix");
       acix_endpoint = endpoint;
     }
+    else if (command == "perflogdir") {
+      perf_log.SetOutput(config_next_arg(rest) + "/data.perflog");
+    }
+    else if (command == "enable_perflog_reporting") {
+      std::string enableperflog = config_next_arg(rest);
+      if (enableperflog == "yes") perf_log.SetEnabled(true);
+    }
   }
   return true;
 }
