@@ -61,14 +61,19 @@ static inline bool read_str(int f,char* buf,int size) {
 
 std::ostream &operator<< (std::ostream &o,const FileData &fd) {
   std::string escaped_pfn(Arc::escape_chars(fd.pfn, " \\\r\n", '\\', false));
-  o.write(escaped_pfn.c_str(), escaped_pfn.size());
-  o.put(' ');
-  std::string escaped_lfn(Arc::escape_chars(fd.lfn, " \\\r\n", '\\', false));
-  o.write(escaped_lfn.c_str(), escaped_lfn.size());
-  if(fd.lfn.empty() || fd.cred.empty()) return o;
-  o.put(' ');
-  std::string escaped_cred(Arc::escape_chars(fd.cred, " \\\r\n", '\\', false));
-  o.write(escaped_cred.c_str(), escaped_cred.size());
+  if(!escaped_pfn.empty()) {
+    o.write(escaped_pfn.c_str(), escaped_pfn.size());
+    std::string escaped_lfn(Arc::escape_chars(fd.lfn, " \\\r\n", '\\', false));
+    if(!escaped_lfn.empty()) {
+      o.put(' ');
+      o.write(escaped_lfn.c_str(), escaped_lfn.size());
+      std::string escaped_cred(Arc::escape_chars(fd.cred, " \\\r\n", '\\', false));
+      if(!escaped_cred.empty()) {
+        o.put(' ');
+        o.write(escaped_cred.c_str(), escaped_cred.size());
+      };
+    };
+  };
   return o;
 }
 
@@ -279,6 +284,10 @@ JobLocalDescription& JobLocalDescription::operator=(const Arc::JobDescription& a
       outputdata.push_back(fdata);
     }
   }
+  // Pick up per job delegation
+  if(!arc_job_desc.DataStaging.DelegationID.empty()) {
+    delegationid = arc_job_desc.DataStaging.DelegationID;
+  };
 
   exec = arc_job_desc.Application.Executable;
   for(std::list<Arc::ExecutableType>::const_iterator e = arc_job_desc.Application.PreExecutable.begin();
@@ -357,6 +366,7 @@ JobLocalDescription& JobLocalDescription::operator=(const Arc::JobDescription& a
 }
 
 const char* const JobLocalDescription::transfersharedefault = "_default";
+int const JobLocalDescription::prioritydefault = 50;
 
 bool LRMSResult::set(const char* s) {
   // 1. Empty string = exit code 0
@@ -492,12 +502,21 @@ bool JobLocalDescription::write(const std::string& fname) const {
   write_pair(f,"delegexpiretime",expiretime);
   write_pair(f,"clientname",clientname);
   write_pair(f,"clientsoftware",clientsoftware);
+  write_pair(f,"delegationid",delegationid);
   write_pair(f,"sessiondir",sessiondir);
   write_pair(f,"diskspace",Arc::tostring(diskspace));
   write_pair(f,"failedstate",failedstate);
   write_pair(f,"failedcause",failedcause);
   write_pair(f,"credentialserver",credentialserver);
   write_pair(f,"freestagein",freestagein);
+  for(std::list<std::string>::const_iterator lv=localvo.begin();
+      lv != localvo.end(); ++lv) {
+    write_pair(f,"localvo",(*lv));
+  };
+  for(std::list<std::string>::const_iterator vf=voms.begin();
+      vf != voms.end(); ++vf) {
+    write_pair(f,"voms",(*vf));
+  };
   for(std::list<std::string>::const_iterator act_id=activityid.begin();
       act_id != activityid.end(); ++act_id) {
     write_pair(f,"activityid",(*act_id));
@@ -528,6 +547,8 @@ bool JobLocalDescription::read(const std::string& fname) {
   char buf[4096];
   std::string name;
   activityid.clear();
+  localvo.clear();
+  voms.clear();
   for(;;) {
     if(!read_str(f,buf,sizeof(buf))) break;;
     name.erase();
@@ -613,11 +634,20 @@ bool JobLocalDescription::read(const std::string& fname) {
     else if(name == "delegexpiretime") { expiretime = buf+p; }
     else if(name == "clientname") { clientname = buf+p; }
     else if(name == "clientsoftware") { clientsoftware = buf+p; }
+    else if(name == "delegationid") { delegationid = buf+p; }
     else if(name == "sessiondir") { sessiondir = buf+p; }
     else if(name == "failedstate") { failedstate = buf+p; }
     else if(name == "failedcause") { failedcause = buf+p; }
     else if(name == "credentialserver") { credentialserver = buf+p; }
     else if(name == "freestagein") { freestagein = read_boolean(buf+p); }
+    else if(name == "localvo") {
+      std::string temp_s(buf+p);
+      localvo.push_back(temp_s);
+    }
+    else if(name == "voms") {
+      std::string temp_s(buf+p);
+      voms.push_back(temp_s);
+    }
     else if(name == "diskspace") {
       std::string temp_s(buf+p);
       unsigned long long int n;
