@@ -41,7 +41,6 @@
 #include "../../../gridftpd/names.h"
 #include "../../../gridftpd/misc.h"
 #include "../../../gridftpd/fileplugin/fileplugin.h"
-#include "../../delegation/DelegationStore.h"
 
 #include "jobplugin.h"
 
@@ -118,7 +117,9 @@ JobPlugin::JobPlugin(std::istream &cfile,userspec_t &user_s,FileNode& node):
     cont_plugins(new ContinuationPlugins),
     cred_plugin(new RunPlugin),
     user_a(user_s.user),
-    job_map(user_s.user) {
+    job_map(user_s.user),
+    matched_vo(NULL),
+    matched_voms(NULL) {
 
   // Because this plugin can load own plugins it's symbols need to 
   // become available in order to avoid duplicate resolution.
@@ -128,10 +129,13 @@ JobPlugin::JobPlugin(std::istream &cfile,userspec_t &user_s,FileNode& node):
   job_rsl_max_size = DEFAULT_JOB_RSL_MAX_SIZE;
   direct_fs=NULL;
   proxy_fname="";
+  proxy_is_deleg=false;
   std::string configfile = user_s.get_config_file();
   readonly=false;
   chosenFilePlugin=NULL;
-  srand(time(NULL)); 
+  matched_vo=user_a.default_group_vo();
+  matched_voms=user_a.default_group_voms();
+  srand(time(NULL) + rand()); 
   for(;;) {
     std::string rest;
     std::string command=config_read_line(cfile,rest);
@@ -228,7 +232,9 @@ JobPlugin::JobPlugin(std::istream &cfile,userspec_t &user_s,FileNode& node):
       dirs.control_dir = config.ControlDir();
       dirs.session_dir = session_dirs.front();
       gm_dirs_info.push_back(dirs);
-      gm_dirs_non_draining.push_back(dirs); // can't drain main control dir
+      if (!session_dirs_non_draining.empty()) {
+        gm_dirs_non_draining.push_back(dirs);
+      }
     }
     if (gm_dirs_info.empty()) {
       logger.msg(Arc::ERROR, "No control or remote control directories defined in configuration");
@@ -288,11 +294,23 @@ JobPlugin::JobPlugin(std::istream &cfile,userspec_t &user_s,FileNode& node):
   if(!initialized) {
     logger.msg(Arc::ERROR, "Job plugin was not initialised");
   }
-  if((!user_a.is_proxy()) ||
-     (user_a.proxy() == NULL) || (user_a.proxy()[0] == 0)) {
-    logger.msg(Arc::WARNING, "No delegated credentials were passed");
-  } else {
+  deleg_db_type = DelegationStore::DbBerkeley;
+  switch(config.DelegationDBType()) {
+   case GMConfig::deleg_db_bdb:
+    deleg_db_type = DelegationStore::DbBerkeley;
+    break;
+   case GMConfig::deleg_db_sqlite:
+    deleg_db_type = DelegationStore::DbSQLite;
+    break;
+  };
+  if(user_a.proxy() != NULL) {
     proxy_fname=user_a.proxy();
+    if((!proxy_fname.empty()) && user_a.is_proxy()) {
+      proxy_is_deleg = true;
+    }
+  }
+  if(!proxy_is_deleg) {
+    logger.msg(Arc::WARNING, "No delegated credentials were passed");
   };
   subject=user_a.DN();
   port=user_s.get_port();
@@ -311,7 +329,7 @@ JobPlugin::JobPlugin(std::istream &cfile,userspec_t &user_s,FileNode& node):
 
 JobPlugin::~JobPlugin(void) {
   delete_job_id();
-  if(proxy_fname.length() != 0) { remove(proxy_fname.c_str()); };
+  if(!proxy_fname.empty()) { remove(proxy_fname.c_str()); };
   if(cont_plugins) delete cont_plugins;
   if(cred_plugin) delete cred_plugin;
   for (unsigned int i = 0; i < file_plugins.size(); i++) {
@@ -417,11 +435,8 @@ int JobPlugin::removedir(std::string &dname) {
     config.SetSessionRoot(sessiondir);
     job_state_t status=job_state_read_file(id,config);
     logger.msg(Arc::INFO, "Cleaning job %s", id);
-    if((status == JOB_STATE_FINISHED) ||
-       (status == JOB_STATE_DELETED)) { /* remove files */
-      if(job_clean_final(GMJob(id,user,sessiondir+"/"+id),config)) return 0;
-    }
-    else { /* put marks */
+    /* put marks because cleaning job may also involve removing locks */
+    {
       GMJob job(id,user);
       bool res = job_cancel_mark_put(job,config);
       res &= job_clean_mark_put(job,config);
@@ -590,6 +605,7 @@ int JobPlugin::open(const char* name,open_modes mode,unsigned long long int size
 int JobPlugin::close(bool eof) {
   if(!initialized || chosenFilePlugin == NULL) return 1;
   if(!rsl_opened) {
+    // file transfer finished
     if((getuid()==0) && config.StrictSession()) {
       SET_USER_UID;
       int r=chosenFilePlugin->close(eof);
@@ -598,6 +614,7 @@ int JobPlugin::close(bool eof) {
     };
     return chosenFilePlugin->close(eof);
   };
+  // Job description/action request transfer finished
   rsl_opened=false;
   if(job_id.length() == 0) {
     error_description="There is no job ID defined.";
@@ -612,6 +629,7 @@ int JobPlugin::close(bool eof) {
   /* analyze rsl (checking, substituting, etc)*/
   JobDescriptionHandler job_desc_handler(config);
   JobLocalDescription job_desc;
+  // Initial parsing of job/action request
   JobReqResult parse_result = job_desc_handler.parse_job_req(job_id,job_desc,true);
   if (parse_result != JobReqSuccess) {
     error_description="Failed to parse job/action description.";
@@ -621,33 +639,40 @@ int JobPlugin::close(bool eof) {
   };
   if(job_desc.action.length() == 0) job_desc.action="request";
   if(job_desc.action == "cancel") {
+    // Request to cancel existing job
     delete_job_id();
     if(job_desc.jobid.length() == 0) {
       error_description="Missing ID in request to cancel job.";
       logger.msg(Arc::ERROR, "%s", error_description);
       return 1;
     };
+    // fall back to RESTful interface
     return removefile(job_desc.jobid);
   };
   if(job_desc.action == "clean") {
+    // Request to remove existing job
     delete_job_id();
     if(job_desc.jobid.length() == 0) {
       error_description="Missing ID in request to clean job.";
       logger.msg(Arc::ERROR, "%s", error_description);
       return 1;
     };
+    // fall back to RESTful interface
     return removedir(job_desc.jobid);
   };
   if(job_desc.action == "renew") {
+    // Request to renew delegated credentials
     delete_job_id();
     if(job_desc.jobid.length() == 0) {
       error_description="Missing ID in request to renew credentials.";
       logger.msg(Arc::ERROR, "%s", error_description);
       return 1;
     };
+    // fall back to RESTful interface
     return checkdir(job_desc.jobid);
   };
   if(job_desc.action == "restart") {
+    // Request to restart failed job
     delete_job_id();
     if(job_desc.jobid.length() == 0) {
       error_description="Missing ID in request to restart job.";
@@ -692,6 +717,7 @@ int JobPlugin::close(bool eof) {
     logger.msg(Arc::ERROR, "action(%s) != request", job_desc.action);
     return 1;
   };
+  // Request for creating new job
   if(readonly) {
     delete_job_id();
     error_description="You are not allowed to submit new jobs to this service.";
@@ -739,9 +765,9 @@ int JobPlugin::close(bool eof) {
         ::close(h_old);
       };
       if(l == -1) {
-        logger.msg(Arc::ERROR, "Failed writing RSL");
+        logger.msg(Arc::ERROR, "Failed writing job description");
         remove(rsl_fname.c_str());
-        error_description="Failed to store job RSL description.";
+        error_description="Failed to store job description.";
         delete_job_id(); return 1;
       };
     };
@@ -800,6 +826,10 @@ int JobPlugin::close(bool eof) {
     delete_job_id(); 
     return 1;
   };
+  // Also pick up global delegation id if any
+  if(!job_desc.delegationid.empty()) {
+    deleg_ids.push_back(job_desc.delegationid);
+  };
   /* ****************************************
    * Start local file                       *
    **************************************** */
@@ -822,42 +852,68 @@ int JobPlugin::close(bool eof) {
   };
   job_desc.headnode=endpoint;
   job_desc.interface="org.nordugrid.gridftpjob";
-  /* ***********************************************
-   * Try to create proxy                           *
-   *********************************************** */
-  if(proxy_fname.length() != 0) {
-    std::string fname=config.ControlDir()+"/job."+job_id+".proxy";
-    int h=::open(fname.c_str(),O_WRONLY | O_CREAT | O_EXCL,0600);
-    if(h == -1) {
-      error_description="Failed to store credentials.";
-      return 1;
-    };
-    int hh=::open(proxy_fname.c_str(),O_RDONLY);
-    if(hh == -1) {
-      ::close(h);
-      ::remove(fname.c_str());
-      error_description="Failed to read credentials.";
-      return 1;
-    };
-    fix_file_owner(fname,job);
-    int l,ll;
-    const char* s;
-    char buf[256]; 
-    for(;;) {
-      ll=::read(hh,buf,sizeof(buf));
-      if((ll==0) || (ll==-1)) break;
-      for(l=0,s=buf;(ll>0) && (l!=-1);s+=l,ll-=l) l=::write(h,s,ll);
-      if(l==-1) break;
-    };
-    ::close(h);
-    ::close(hh);
-    try {
-      Arc::Credential ci(proxy_fname, proxy_fname, config.CertDir(), "");
-      job_desc.expiretime = ci.GetEndTime();
-    } catch (std::exception&) {
-      job_desc.expiretime = time(NULL);
+  if(matched_vo != NULL) {
+    job_desc.localvo.push_back(matched_vo);
+  };
+  if(matched_voms != NULL) {
+    for(std::vector<voms_fqan_t>::const_iterator f = matched_voms->fqans.begin();
+                           f != matched_voms->fqans.end(); ++f) {
+      std::string fqan;
+      f->str(fqan);
+      job_desc.voms.push_back(fqan);
     };
   };
+
+
+  /* ***********************************************
+   * Try to create delegation and proxy file       *
+   *********************************************** */
+  if(!proxy_fname.empty()) {
+    std::string proxy_data;
+    (void)Arc::FileRead(proxy_fname, proxy_data);
+    if(!proxy_data.empty()) {
+      if(proxy_is_deleg && job_desc.delegationid.empty()) {
+        // If we have gridftp delegation and no other generic delegation provided - store it
+        ARex::DelegationStore deleg(config.DelegationDir(), deleg_db_type, false);
+        std::string deleg_id;
+        if(!deleg.AddCred(deleg_id, subject, proxy_data)) {
+          error_description="Failed to store delegation.";
+          logger.msg(Arc::ERROR, "%s", error_description);
+          delete_job_id(); 
+          return 1;
+        };
+        job_desc.delegationid = deleg_id;
+        deleg_ids.push_back(deleg_id); // one more delegation id
+      };
+#if 0
+      // And store public credentials into proxy file
+      try {
+        Arc::Credential ci(proxy_fname, proxy_fname, config.CertDir(), "");
+        job_desc.expiretime = ci.GetEndTime();
+        std::string user_cert;
+        ci.OutputCertificate(user_cert);
+        ci.OutputCertificateChain(user_cert);
+        if(!job_proxy_write_file(job,config,user_cert)) {
+          error_description="Failed to store user credentials.";
+          logger.msg(Arc::ERROR, "%s", error_description);
+          delete_job_id(); 
+          return 1;
+        };
+      } catch (std::exception&) {
+        job_desc.expiretime = time(NULL);
+      };
+#else
+      // For backward compatibility during conversion time
+      // store full proxy into proxy file
+      if(!job_proxy_write_file(job,config,proxy_data)) {
+        error_description="Failed to store user credentials.";
+        logger.msg(Arc::ERROR, "%s", error_description);
+        delete_job_id(); 
+        return 1;
+      };
+#endif
+    };
+  }
   /* ******************************************
    * Write local file                         *
    ****************************************** */
@@ -939,16 +995,6 @@ int JobPlugin::close(bool eof) {
     return 1;
   }
 
-  // Put lock on delegated credentials
-  if(!deleg_ids.empty()) {
-    if(!ARex::DelegationStore(config.DelegationDir(),false).LockCred(job_id,deleg_ids,subject)) {
-      logger.msg(Arc::ERROR, "Failed to lock delegated credentials");
-      delete_job_id();
-      error_description="Failed to lock delegated credentials.";
-      return 1;
-    };
-  }
-
   /* **********************************************************
    * Create status file (do it last so GM picks job up here)  *
    ********************************************************** */
@@ -959,6 +1005,19 @@ int JobPlugin::close(bool eof) {
     return 1;
   };
 
+  // Put lock on delegated credentials
+  // Can do that after creating status file because delegations are 
+  // fresh and hence won't be deleted while locking.
+  if(!deleg_ids.empty()) {
+    ARex::DelegationStore store(config.DelegationDir(),deleg_db_type,false);
+    if(!store.LockCred(job_id,deleg_ids,subject)) {
+      logger.msg(Arc::ERROR, "Failed to lock delegated credentials: %s", store.GetFailure());
+      delete_job_id();
+      error_description="Failed to lock delegated credentials.";
+      return 1;
+    };
+  }
+
   SignalFIFO(config.ControlDir());
   job_id.resize(0);
   chosenFilePlugin = NULL;
@@ -1172,7 +1231,7 @@ int JobPlugin::checkdir(std::string &dirname) {
     };
     return 0;
   };
-  if((dirname == id) && (proxy_fname.length())) {  /* cd to session directory - renew proxy request */
+  if((dirname == id) && (!proxy_fname.empty()) && proxy_is_deleg) {  /* cd to session directory - renew proxy request */
     JobLocalDescription job_desc;
     if(!job_local_read_file(id,config,job_desc)) {
       error_description="Job is probably corrupted: can't read internal information.";
@@ -1183,21 +1242,39 @@ int JobPlugin::checkdir(std::string &dirname) {
     std::string old_proxy_fname=config.ControlDir()+"/job."+id+".proxy";
     Arc::Time new_proxy_expires;
     Arc::Time old_proxy_expires;
+    std::string proxy_data;
+    std::string user_cert;
     try {
-      Arc::Credential new_ci(proxy_fname, proxy_fname, config.CertDir(), "");
+      (void)Arc::FileRead(proxy_fname, proxy_data);
+      if(proxy_data.empty()) {
+        error_description="Failed to obtain delegation content.";
+        logger.msg(Arc::ERROR, "%s", error_description);
+        return 1;
+      };
+      Arc::Credential new_ci(proxy_data, proxy_data, config.CertDir(), "", "", false);
       new_proxy_expires = new_ci.GetEndTime();
+      new_ci.OutputCertificate(user_cert);
+      new_ci.OutputCertificateChain(user_cert);
     } catch (std::exception&) { };
     try {
-      Arc::Credential old_ci(old_proxy_fname, old_proxy_fname, config.CertDir(), "");
+      Arc::Credential old_ci(old_proxy_fname, "", config.CertDir(), "");
       old_proxy_expires = old_ci.GetEndTime();
     } catch (std::exception&) { };
     if(new_proxy_expires > old_proxy_expires) {
       /* try to renew proxy */
       logger.msg(Arc::INFO, "Renewing proxy for job %s", id);
-      if(renew_proxy(old_proxy_fname.c_str(),proxy_fname.c_str()) == 0) {
-        fix_file_owner(old_proxy_fname,user);
+      ARex::DelegationStore deleg(config.DelegationDir(),deleg_db_type,false);
+      if((!job_desc.delegationid.empty()) && deleg.PutCred(job_desc.delegationid, subject, proxy_data)) {
+        // Also store public content into job.#.proxy
+        // Ignore error because main store is already updated
+        GMJob job(id, user, "", JOB_STATE_ACCEPTED);
+#if 0
+        (void)job_proxy_write_file(job, config, user_cert);
+#else
+        // For backward compatibility during transitional period store whole proxy
+        (void)job_proxy_write_file(job, config, proxy_data);
+#endif
         logger.msg(Arc::INFO, "New proxy expires at %s", Arc::TimeStamp(Arc::Time(new_proxy_expires), Arc::UserTime));
-        GMJob job(id,user,"",JOB_STATE_ACCEPTED);
         job_desc.expiretime=new_proxy_expires;
         if(!job_local_write_file(job,config,job_desc)) {
           logger.msg(Arc::ERROR, "Failed to write 'local' information");
@@ -1205,20 +1282,6 @@ int JobPlugin::checkdir(std::string &dirname) {
         error_description="Applying external credentials locally failed.";
         ApplyLocalCred(user,config,&id,"renew");
         error_description="";
-        /* Cause restart of job if it potentially failed
-           because of expired proxy */
-        if((old_proxy_expires < Arc::Time()) && (
-            (job_desc.failedstate ==
-                  GMJob::get_state_name(JOB_STATE_PREPARING)) ||
-            (job_desc.failedstate ==
-                  GMJob::get_state_name(JOB_STATE_FINISHING))
-           )
-          ) {
-          logger.msg(Arc::INFO, "Job could have died due to expired proxy: restarting");
-          if(!job_restart_mark_put(GMJob(id,user),config)) {
-            logger.msg(Arc::ERROR, "Failed to report renewed proxy to job");
-          };
-        };
       } else {
         logger.msg(Arc::ERROR, "Failed to renew proxy");
       };
