@@ -13,6 +13,8 @@
 #include "../conf/UrlMapConfig.h"
 #include "../files/ControlFileHandling.h"
 #include "../conf/StagingConfig.h"
+#include "../../delegation/DelegationStore.h"
+#include "../../delegation/DelegationStores.h"
 
 #include "GMJob.h"
 
@@ -31,8 +33,10 @@ void DTRInfo::receiveDTR(DataStaging::DTR_ptr dtr) {
 
 Arc::Logger DTRGenerator::logger(Arc::Logger::getRootLogger(), "Generator");
 
-bool compare_job_description(GMJob first, GMJob second) {
-  return first.get_local()->priority > second.get_local()->priority;
+bool compare_job_description(GMJob const& first, GMJob const& second) {
+  int priority_first = first.GetLocalDescription() ? first.GetLocalDescription()->priority : JobLocalDescription::prioritydefault;
+  int priority_second = first.GetLocalDescription() ? second.GetLocalDescription()->priority : JobLocalDescription::prioritydefault;
+  return priority_first > priority_second;
 }
 
 void DTRGenerator::main_thread(void* arg) {
@@ -54,6 +58,16 @@ void DTRGenerator::thread() {
     event_lock.lock();
     std::list<std::string>::iterator it_cancel = jobs_cancelled.begin();
     while (it_cancel != jobs_cancelled.end()) {
+      // check if it is still in received queue
+      std::list<GMJob>::iterator it_jobs = jobs_received.begin();
+      for(; it_jobs != jobs_received.end(); ++it_jobs) {
+        if(*it_jobs == *it_cancel) break;
+      };
+      if(it_jobs != jobs_received.end()) {
+        jobs_received.erase(it_jobs);
+        continue;
+      };
+      // job must be in scheduler already
       event_lock.unlock();
       processCancelledJob(*it_cancel);
       event_lock.lock();
@@ -68,7 +82,6 @@ void DTRGenerator::thread() {
       event_lock.lock();
       // delete DTR LogDestinations
       (*it_dtrs)->get_logger()->deleteDestinations();
-
       it_dtrs = dtrs_received.erase(it_dtrs);
     }
 
@@ -168,6 +181,9 @@ DTRGenerator::DTRGenerator(const GMConfig& config,
   // Limit on remote delivery size
   scheduler->SetRemoteSizeLimit(staging_conf.remote_size_limit);
 
+  // Set performance metrics logging
+  scheduler->SetJobPerfLog(staging_conf.perf_log);
+
   // End of configuration - start Scheduler thread
   scheduler->start();
 
@@ -349,7 +365,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   GMJob job(jobid, dtr->get_local_user(), session_dir);
   std::string dtr_transfer_statistics;
   
-  if (dtr->error() && dtr->get_status() != DataStaging::DTRStatus::CANCELLED) {
+  if (dtr->error() && dtr->is_mandatory() && dtr->get_status() != DataStaging::DTRStatus::CANCELLED) {
     // for uploads, report error but let other transfers continue
     // for downloads, cancel all other transfers
     logger.msg(Arc::ERROR, "%s: DTR %s to copy file %s failed",
@@ -368,6 +384,10 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   else if (dtr->get_status() != DataStaging::DTRStatus::CANCELLED) {
     // remove from job.id.input/output files on success
     // find out if download or upload by checking which is remote file
+    if (dtr->error() && !dtr->is_mandatory()) {
+      dtr->get_logger()->msg(Arc::INFO, "%s: DTR %s to copy to %s failed but is not mandatory",
+                             jobid, dtr->get_id(), dtr->get_destination_str());
+    }
     std::list<FileData> files;
 
     if (dtr->get_source()->Local()) {
@@ -505,7 +525,9 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   // finished, which results in job being submitted to DTR again
   active_dtrs.insert(std::pair<std::string, std::string>(jobid, dtr->get_id()));
 
-  bool finished_with_error = (finished_jobs.find(jobid) != finished_jobs.end() && !finished_jobs[jobid].empty());
+  bool finished_with_error = ((finished_jobs.find(jobid) != finished_jobs.end() &&
+                               !finished_jobs[jobid].empty()) ||
+                              dtr->get_status() == DataStaging::DTRStatus::CANCELLED);
   lock.unlock();
 
   if (dtr->get_source()->Local()) {
@@ -573,7 +595,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
 }
 
 
-bool DTRGenerator::processReceivedJob(const GMJob& job) {
+bool DTRGenerator::processReceivedJob(GMJob& job) {
 
   JobId jobid(job.get_id());
   logger.msg(Arc::VERBOSE, "%s: Received data staging request to %s files", jobid,
@@ -581,7 +603,25 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
 
   uid_t job_uid = config.StrictSession() ? job.get_user().get_uid() : 0;
   uid_t job_gid = config.StrictSession() ? job.get_user().get_gid() : 0;
-  
+
+  // Default credentials to be used by transfering files if not specified per file
+  std::string default_cred = job_proxy_filename(jobid, config); // TODO: drop job.proxy as source of delegation
+  {
+    JobLocalDescription job_desc;
+    if(job_local_read_file(jobid, config, job_desc)) {
+      if(!job_desc.delegationid.empty()) {
+        ARex::DelegationStores* delegs = config.Delegations();
+        if(delegs) {
+          DelegationStore& deleg = delegs->operator[](config.DelegationDir());
+          std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN);
+          if(!fname.empty()) {
+            default_cred = fname;
+          };
+        };
+      };
+    };
+  };
+
   // Create a file for the transfer statistics and fix its permissions
   std::string fname = config.ControlDir() + "/job." + jobid + ".statistics";
   std::ofstream f(fname.c_str(),std::ios::out | std::ios::app);
@@ -657,13 +697,19 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
     // resolve directories
     for (it = files.begin(); it != files.end() ;) {
       if (it->pfn.find("@") == 1) { // GM puts a slash on the front of the local file
+        // Following is always empty currently. But it will start working as soon as 
+        // there is a way to pass credentials for dynamic files. But so far default_cred
+        // is always picked up.
+        std::string cred(it->cred);
+        if(cred.empty()) cred = default_cred;
+        std::list<FileData> files_;
         std::string outputfilelist = job.SessionDir() + std::string("/") + it->pfn.substr(2);
         logger.msg(Arc::INFO, "%s: Reading output files from user generated list in %s", jobid, outputfilelist);
-        if (!job_Xput_read_file(outputfilelist, files, job_uid, job_gid)) {
+        if (!job_Xput_read_file(outputfilelist, files_, job_uid, job_gid)) {
           logger.msg(Arc::ERROR, "%s: Error reading user generated output file list in %s", jobid, outputfilelist);
           lock.lock();
           // Only write this failure if no previous failure
-          if (job.GetFailure(config).empty()) {
+          if (!job.CheckFailure(config)) {
             finished_jobs[jobid] = std::string("Error reading user generated output file list");
           } else {
             finished_jobs[jobid] = "";
@@ -673,6 +719,11 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
           if (kicker_func) (*kicker_func)(kicker_arg);
           return false;
         }
+        // Attach dynamic files and assign credentials to them unless already available
+        for(std::list<FileData>::iterator it_ = files_.begin(); it_ != files_.end(); ++it_) {
+          if(it_->cred.empty()) it_->cred = cred;
+          files.push_back(*it_);
+        }
         it->pfn.erase(1, 1);
         ++it;
         continue;
@@ -686,7 +737,7 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
             logger.msg(Arc::ERROR, "%s: Failed to list output directory %s: %s", jobid, dir, Arc::StrError(errno));
             lock.lock();
             // Only write this failure if no previous failure
-            if (job.GetFailure(config).empty()) {
+            if (!job.CheckFailure(config)) {
               finished_jobs[jobid] = std::string("Failed to list output directory");
             } else {
               finished_jobs[jobid] = "";
@@ -866,8 +917,8 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
       usercfg.ProxyPath(i->cred);
       if (Arc::FileRead(i->cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
     } else {
-      usercfg.ProxyPath(job_proxy_filename(jobid, config));
-      if (Arc::FileRead(job_proxy_filename(jobid, config), proxy_cred)) usercfg.CredentialString(proxy_cred);
+      usercfg.ProxyPath(default_cred);
+      if (Arc::FileRead(default_cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
     }
     // logger for these DTRs. LogDestinations should be deleted when DTR is received back
     DataStaging::DTRLogger dtr_log(new Arc::Logger(Arc::Logger::getRootLogger(), "DataStaging.DTR"));
@@ -885,8 +936,8 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
     // set sub-share for download or upload
     dtr->set_sub_share((job.get_state() == JOB_STATE_PREPARING) ? "download" : "upload");
     // set priority as given in job description
-    if (job.get_local())
-      dtr->set_priority(job.get_local()->priority);
+    if (job.GetLocalDescription(config))
+      dtr->set_priority(job.GetLocalDescription(config)->priority);
     // set whether to use A-REX host certificate for remote delivery services
     dtr->host_cert_for_remote_delivery(staging_conf.use_host_cert_for_remote_delivery);
     // set real location if ACIX is used
@@ -894,6 +945,8 @@ bool DTRGenerator::processReceivedJob(const GMJob& job) {
       dtr->get_source()->AddLocation(Arc::URL(original_source), Arc::URL(original_source).ConnectionURL());
       dtr->set_use_acix(true);
     }
+    dtr->get_job_perf_log().SetOutput(staging_conf.perf_log.GetOutput());
+    dtr->get_job_perf_log().SetEnabled(staging_conf.perf_log.GetEnabled());
 
     DataStaging::DTRCacheParameters cache_parameters;
     CacheConfig cache_params(config.CacheParams());
@@ -950,8 +1003,10 @@ int DTRGenerator::checkUploadedFiles(GMJob& job) {
   uid_t job_gid = config.StrictSession() ? job.get_user().get_gid() : 0;
 
   std::string session_dir;
-  if (job.get_local() && !job.get_local()->sessiondir.empty()) session_dir = job.get_local()->sessiondir;
-  else session_dir = config.SessionRoot(jobid) + '/' + jobid;
+  if (job.GetLocalDescription(config) && !job.GetLocalDescription(config)->sessiondir.empty())
+    session_dir = job.GetLocalDescription(config)->sessiondir;
+  else
+    session_dir = config.SessionRoot(jobid) + '/' + jobid;
   // get input files list
   std::list<std::string> uploaded_files;
   std::list<std::string>* uploaded_files_ = NULL;
