@@ -6,6 +6,7 @@
 #include <fcntl.h>
 
 #include <arc/ArcLocation.h>
+#include <arc/JobPerfLog.h>
 #include <arc/credential/VOMSUtil.h>
 
 #include "../files/ControlFileHandling.h"
@@ -53,6 +54,42 @@ bool JobsList::AddJobNoCheck(const JobId &id,uid_t uid,gid_t gid){
   return AddJobNoCheck(id,i,uid,gid);
 }
 
+void JobsList::UpdateJobCredentials(JobsList::iterator &i) {
+  if(GetLocalDescription(i)) {
+    std::string delegation_id = i->local->delegationid;
+    if(!delegation_id.empty()) {
+      ARex::DelegationStores* delegs = config.delegations;
+      if(delegs) {
+        std::string cred;
+        if((*delegs)[config.DelegationDir()].GetCred(delegation_id,i->local->DN,cred)) {
+          (void)job_proxy_write_file(*i,config,cred);
+        };
+      };
+    };
+  };
+}
+
+void JobsList::SetJobState(JobsList::iterator &i, job_state_t new_state, const char* reason) {
+  if(i->job_state != new_state) {
+    std::string msg = Arc::Time().str(Arc::UTCTime);
+    msg += " Job state change "; 
+    msg += i->get_state_name();
+    msg += " -> ";
+    msg += GMJob::get_state_name(new_state);
+    if(reason) {
+      msg += "   Reason: ";
+      msg += reason;
+    };
+    msg += "\n";
+    i->job_state = new_state;
+    job_errors_mark_add(*i,config,msg);
+    // During intermediate period job.proxy file must contain full delegated proxy.
+    // To ensure its content is up to date even if proxy was updated in store here
+    // we update content of that file on every job state change.
+    UpdateJobCredentials(i);
+  };
+}
+
 bool JobsList::AddJobNoCheck(const JobId &id,JobsList::iterator &i,uid_t uid,gid_t gid){
   i=jobs.insert(jobs.end(),GMJob(id, Arc::User(uid)));
   i->keep_finished=config.keep_finished;
@@ -60,7 +97,7 @@ bool JobsList::AddJobNoCheck(const JobId &id,JobsList::iterator &i,uid_t uid,gid
   if (!GetLocalDescription(i)) {
     // safest thing to do is add failure and move to FINISHED
     i->AddFailure("Internal error");
-    i->job_state = JOB_STATE_FINISHED;
+    SetJobState(i, JOB_STATE_FINISHED, "Internal failure");
     FailedJob(i, false);
     if(!job_state_write_file(*i,config,i->job_state)) {
       logger.msg(Arc::ERROR, "%s: Failed reading .local and changing state, job and "
@@ -107,6 +144,7 @@ void JobsList::PrepareToDestroy(void) {
   }
 }
 
+
 bool JobsList::ActJobs(void) {
 
   bool res = true;
@@ -135,16 +173,16 @@ bool JobsList::DestroyJob(JobsList::iterator &i,bool finished,bool active) {
   logger.msg(Arc::INFO,"%s: Destroying",i->job_id);
   job_state_t new_state=i->job_state;
   if(new_state == JOB_STATE_UNDEFINED) {
+    // Try to obtain real job state
     if((new_state=job_state_read_file(i->job_id,config))==JOB_STATE_UNDEFINED) {
       logger.msg(Arc::ERROR,"%s: Can't read state - no comments, just cleaning",i->job_id);
       UnlockDelegation(i);
       job_clean_final(*i,config);
-      if(i->local) delete i->local;
       i=jobs.erase(i);
       return true;
     }
+    i->job_state = new_state;
   }
-  i->job_state = new_state;
   if((new_state == JOB_STATE_FINISHED) && (!finished)) { ++i; return true; }
   if(!active) { ++i; return true; }
   if((new_state != JOB_STATE_INLRMS) || 
@@ -152,7 +190,6 @@ bool JobsList::DestroyJob(JobsList::iterator &i,bool finished,bool active) {
     logger.msg(Arc::INFO,"%s: Cleaning control and session directories",i->job_id);
     UnlockDelegation(i);
     job_clean_final(*i,config);
-    if(i->local) delete i->local;
     i=jobs.erase(i);
     return true;
   }
@@ -162,7 +199,6 @@ bool JobsList::DestroyJob(JobsList::iterator &i,bool finished,bool active) {
     logger.msg(Arc::WARNING,"%s: Cancellation failed (probably job finished) - cleaning anyway",i->job_id);
     UnlockDelegation(i);
     job_clean_final(*i,config);
-    if(i->local) delete i->local;
     i=jobs.erase(i);
     return true;
   }
@@ -170,7 +206,6 @@ bool JobsList::DestroyJob(JobsList::iterator &i,bool finished,bool active) {
   logger.msg(Arc::INFO,"%s: Cancellation probably succeeded - cleaning",i->job_id);
   UnlockDelegation(i);
   job_clean_final(*i,config);
-  if(i->local) delete i->local;
   i=jobs.erase(i);
   return true;
 }
@@ -250,18 +285,12 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
     // no child was running yet, or recovering from fault 
     // write grami file for submit-X-job
     // TODO: read existing grami file to check if job is already submitted
-    JobLocalDescription* job_desc;
-    if(i->local) { job_desc=i->local; }
-    else {
-      job_desc=new JobLocalDescription;
-      if(!job_local_read_file(i->job_id,config,*job_desc)) {
-        logger.msg(Arc::ERROR,"%s: Failed reading local information",i->job_id);
-        if(!cancel) i->AddFailure("Internal error: can't read local file");
-        delete job_desc;
-        return false;
-      }
-      i->local=job_desc;
-    }
+    if(!(i->GetLocalDescription(config))) {
+      logger.msg(Arc::ERROR,"%s: Failed reading local information",i->job_id);
+      if(!cancel) i->AddFailure("Internal error: can't read local file");
+      return false;
+    };
+    JobLocalDescription* job_desc = i->local;
     if(!cancel) {  // in case of cancel all preparations are already done
       const char *local_transfer_s = NULL;
       if(staging_config.get_local_transfer()) {
@@ -447,7 +476,7 @@ bool JobsList::state_loading(const JobsList::iterator &i,bool &state_changed,boo
     return true;
   }
   // if job has already failed then do not set failed state again if DTR failed
-  bool already_failed = !i->GetFailure(config).empty();
+  bool already_failed = i->CheckFailure(config);
   // queryJobFinished() calls i->AddFailure() if any DTR failed
   if (dtr_generator->queryJobFinished(*i)) {
 
@@ -455,7 +484,7 @@ bool JobsList::state_loading(const JobsList::iterator &i,bool &state_changed,boo
     bool result = true;
 
     // check for failure
-    if (!i->GetFailure(config).empty()) {
+    if (i->CheckFailure(config)) {
       if (!already_failed) JobFailStateRemember(i, (up ? JOB_STATE_FINISHING : JOB_STATE_PREPARING));
       result = false;
     }
@@ -495,19 +524,18 @@ job_state_t JobsList::JobFailStateGet(const JobsList::iterator &i) {
     return JOB_STATE_UNDEFINED;
   }
   if(i->local->failedstate.empty()) { return JOB_STATE_UNDEFINED; }
-  for(int n = 0;states_all[n].name != NULL;n++) {
-    if(i->local->failedstate == states_all[n].name) {
-      if(i->local->reruns <= 0) {
-        logger.msg(Arc::ERROR,"%s: Job is not allowed to be rerun anymore",i->job_id);
-        job_local_write_file(*i,config,*(i->local));
-        return JOB_STATE_UNDEFINED;
-      }
-      i->local->failedstate="";
-      i->local->failedcause="";
-      i->local->reruns--;
+  job_state_t state = GMJob::get_state(i->local->failedstate.c_str());
+  if(state != JOB_STATE_UNDEFINED) {
+    if(i->local->reruns <= 0) {
+      logger.msg(Arc::ERROR,"%s: Job is not allowed to be rerun anymore",i->job_id);
       job_local_write_file(*i,config,*(i->local));
-      return states_all[n].id;
+      return JOB_STATE_UNDEFINED;
     }
+    i->local->failedstate="";
+    i->local->failedcause="";
+    i->local->reruns--;
+    job_local_write_file(*i,config,*(i->local));
+    return state;
   }
   logger.msg(Arc::ERROR,"%s: Job failed in unknown state. Won't rerun.",i->job_id);
   i->local->failedstate="";
@@ -582,15 +610,9 @@ bool JobsList::RecreateTransferLists(const JobsList::iterator &i) {
 }
 
 bool JobsList::JobFailStateRemember(const JobsList::iterator &i,job_state_t state,bool internal) {
-  if(!(i->local)) {
-    JobLocalDescription *job_desc = new JobLocalDescription;
-    if(!job_local_read_file(i->job_id,config,*job_desc)) {
-      logger.msg(Arc::ERROR,"%s: Failed reading local information",i->job_id);
-      delete job_desc; return false;
-    }
-    else {
-      i->local=job_desc;
-    }
+  if(!(i->GetLocalDescription(config))) {
+    logger.msg(Arc::ERROR,"%s: Failed reading local information",i->job_id);
+    return false;
   }
   if(state == JOB_STATE_UNDEFINED) {
     i->local->failedstate="";
@@ -598,7 +620,7 @@ bool JobsList::JobFailStateRemember(const JobsList::iterator &i,job_state_t stat
     return job_local_write_file(*i,config,*(i->local));
   }
   if(i->local->failedstate.empty()) {
-    i->local->failedstate=states_all[state].name;
+    i->local->failedstate=GMJob::get_state_name(state);
     i->local->failedcause=internal?"internal":"client";
     return job_local_write_file(*i,config,*(i->local));
   }
@@ -640,7 +662,7 @@ void JobsList::ActJobUndefined(JobsList::iterator &i,
           // to maintain limits properly after restart. Except FINISHED
           // jobs because they are not kept in memory and should be 
           // processed immediately.
-          i->job_state = new_state; // this can be any state, after A-REX restart
+          SetJobState(i, new_state, "(Re)Accepting new job"); // this can be any state, after A-REX restart
           if(new_state == JOB_STATE_ACCEPTED) {
             state_changed = true; // to trigger email notification, etc.
             // first phase of job - just  accepted - parse request
@@ -667,7 +689,6 @@ void JobsList::ActJobUndefined(JobsList::iterator &i,
                 GMJob::get_state_name(new_state),i->get_user().get_uid(),i->get_user().get_gid());
             // Make it clean state after restart
             job_state_write_file(*i,config,i->job_state);
-            i->retries = staging_config.get_max_retries();
             i->Start();
 
             // add to DN map
@@ -706,7 +727,7 @@ void JobsList::ActJobAccepted(JobsList::iterator &i,
           return;
         }
         // check for user specified time
-        if(i->retries == 0 && i->local->processtime != -1 && (i->local->processtime) > time(NULL)) {
+        if(i->local->processtime != -1 && (i->local->processtime) > time(NULL)) {
           logger.msg(Arc::INFO,"%s: State: ACCEPTED: has process time %s",i->job_id.c_str(),
               i->local->processtime.str(Arc::UserTime));
           return;
@@ -715,18 +736,14 @@ void JobsList::ActJobAccepted(JobsList::iterator &i,
         jobs_dn[i->local->DN]++;
         logger.msg(Arc::INFO,"%s: State: ACCEPTED: moving to PREPARING",i->job_id);
         state_changed=true; once_more=true;
-        i->job_state = JOB_STATE_PREPARING;
-        // if first pass then reset retries
-        if (i->retries == 0) i->retries = staging_config.get_max_retries();
+        SetJobState(i, JOB_STATE_PREPARING, "Starting job processing");
         i->Start();
 
         // gather some frontend specific information for user, do it only once
         // Runs user-supplied executable placed at "frontend-info-collector"
-        if(state_changed && i->retries == staging_config.get_max_retries()) {
-          std::string cmd = Arc::ArcLocation::GetToolsDir()+"/frontend-info-collector";
-          char const * const args[2] = { cmd.c_str(), NULL };
-          job_controldiag_mark_put(*i,config,args);
-        }
+        std::string cmd = Arc::ArcLocation::GetToolsDir()+"/frontend-info-collector";
+        char const * const args[2] = { cmd.c_str(), NULL };
+        job_controldiag_mark_put(*i,config,args);
 }
 
 void JobsList::ActJobPreparing(JobsList::iterator &i,
@@ -765,22 +782,20 @@ void JobsList::ActJobPreparing(JobsList::iterator &i,
               JobPending(i);
             } else if(i->local->exec.size() > 0) {
               if((config.max_jobs_running==-1) || (RunningJobs()<config.max_jobs_running)) {
-                i->job_state = JOB_STATE_SUBMITTING;
+                SetJobState(i, JOB_STATE_SUBMITTING, "Pre-staging finished, passing job to LRMS");
                 state_changed=true; once_more=true;
-                i->retries = staging_config.get_max_retries();
               } else {
                 state_changed=false;
                 JobPending(i);
               }
             } else {
-              i->job_state = JOB_STATE_FINISHING;
+              SetJobState(i, JOB_STATE_FINISHING, "Job does NOT define executable. Going directly to post-staging.");
               state_changed=true; once_more=true;
-              i->retries = staging_config.get_max_retries();
             }
           }
         } 
         else {
-          if(i->GetFailure(config).empty()) {
+          if(!i->CheckFailure(config)) {
             i->AddFailure("Data download failed");
           }
           job_error=true;
@@ -794,7 +809,7 @@ void JobsList::ActJobSubmitting(JobsList::iterator &i,
         logger.msg(Arc::VERBOSE,"%s: State: SUBMIT",i->job_id);
         if(state_submitting(i,state_changed)) {
           if(state_changed) {
-            i->job_state = JOB_STATE_INLRMS;
+            SetJobState(i, JOB_STATE_INLRMS, "Job is passed to LRMS");
             once_more=true;
           }
         } else {
@@ -809,7 +824,7 @@ void JobsList::ActJobCanceling(JobsList::iterator &i,
         logger.msg(Arc::VERBOSE,"%s: State: CANCELING",i->job_id);
         if(state_submitting(i,state_changed,true)) {
           if(state_changed) {
-            i->job_state = JOB_STATE_FINISHING;
+            SetJobState(i, JOB_STATE_FINISHING, "Job cancelation succeeded");
             once_more=true;
           }
         }
@@ -828,33 +843,25 @@ void JobsList::ActJobInlrms(JobsList::iterator &i,
           job_error=true;
           return; // go to next job
         }
-        // only check lrms job status on first pass
-        if(i->retries == 0 || i->retries == staging_config.get_max_retries()) {
-          if(i->job_pending || job_lrms_mark_check(i->job_id,config)) {
-            if(!i->job_pending) {
-              logger.msg(Arc::INFO,"%s: Job finished",i->job_id);
-              job_diagnostics_mark_move(*i,config);
-              LRMSResult ec = job_lrms_mark_read(i->job_id,config);
-              if(ec.code() != i->local->exec.successcode) {
-                logger.msg(Arc::INFO,"%s: State: INLRMS: exit message is %i %s",i->job_id,ec.code(),ec.description());
-                i->AddFailure("LRMS error: ("+
-                      Arc::tostring(ec.code())+") "+ec.description());
-                job_error=true;
-                JobFailStateRemember(i,JOB_STATE_INLRMS);
-                // This does not require any special postprocessing and
-                // can go to next state directly
-                state_changed=true; once_more=true;
-                return;
-              }
+        if(i->job_pending || job_lrms_mark_check(i->job_id,config)) {
+          if(!i->job_pending) {
+            logger.msg(Arc::INFO,"%s: Job finished",i->job_id);
+            job_diagnostics_mark_move(*i,config);
+            LRMSResult ec = job_lrms_mark_read(i->job_id,config);
+            if(ec.code() != i->local->exec.successcode) {
+              logger.msg(Arc::INFO,"%s: State: INLRMS: exit message is %i %s",i->job_id,ec.code(),ec.description());
+              i->AddFailure("LRMS error: ("+
+                    Arc::tostring(ec.code())+") "+ec.description());
+              job_error=true;
+              JobFailStateRemember(i,JOB_STATE_INLRMS);
+              // This does not require any special postprocessing and
+              // can go to next state directly
+              state_changed=true; once_more=true;
+              return;
             }
-            state_changed=true; once_more=true;
-            i->job_state = JOB_STATE_FINISHING;
-            // if first pass then reset retries
-            if (i->retries == 0) i->retries = staging_config.get_max_retries();
           }
-        } else {
           state_changed=true; once_more=true;
-          i->job_state = JOB_STATE_FINISHING;
+          SetJobState(i, JOB_STATE_FINISHING, "Job finished executing in LRMS");
         }
 }
 
@@ -866,7 +873,7 @@ void JobsList::ActJobFinishing(JobsList::iterator &i,
         logger.msg(Arc::VERBOSE,"%s: State: FINISHING",i->job_id);
         if(state_loading(i,state_changed,true)) {
           if(state_changed) {
-            i->job_state = JOB_STATE_FINISHED;
+            SetJobState(i, JOB_STATE_FINISHED, "Stage-out finished.");
             if(GetLocalDescription(i)) {
               if (--(jobs_dn[i->local->DN]) <= 0) jobs_dn.erase(i->local->DN);
             }
@@ -878,7 +885,7 @@ void JobsList::ActJobFinishing(JobsList::iterator &i,
         } else {
           state_changed=true; // to send mail
           once_more=true;
-          if(i->GetFailure(config).empty()) {
+          if(!i->CheckFailure(config)) {
             i->AddFailure("Data upload failed");
           }
           job_error=true;
@@ -906,7 +913,7 @@ void JobsList::ActJobFinished(JobsList::iterator &i,
           if(state_ == JOB_STATE_PREPARING) {
             if(RecreateTransferLists(i)) {
               job_failed_mark_remove(i->job_id,config);
-              i->job_state = JOB_STATE_ACCEPTED;
+              SetJobState(i, JOB_STATE_ACCEPTED, "Request to restart failed job");
               JobPending(i); // make it go to end of state immediately
               return;
             }
@@ -916,9 +923,9 @@ void JobsList::ActJobFinished(JobsList::iterator &i,
               job_failed_mark_remove(i->job_id,config);
               if(i->local->downloads > 0) {
                 // missing input files has to be re-downloaded
-                i->job_state = JOB_STATE_ACCEPTED;
+                SetJobState(i, JOB_STATE_ACCEPTED, "Request to restart failed job (some input files are missing)");
               } else {
-                i->job_state = JOB_STATE_PREPARING;
+                SetJobState(i, JOB_STATE_PREPARING, "Request to restart failed job (no input files are missing)");
               }
               JobPending(i); // make it go to end of state immediately
               return;
@@ -926,7 +933,7 @@ void JobsList::ActJobFinished(JobsList::iterator &i,
           } else if(state_ == JOB_STATE_FINISHING) {
             if(RecreateTransferLists(i)) {
               job_failed_mark_remove(i->job_id,config);
-              i->job_state = JOB_STATE_INLRMS;
+              SetJobState(i, JOB_STATE_INLRMS, "Request to restart failed job");
               JobPending(i); // make it go to end of state immediately
               return;
             }
@@ -966,7 +973,7 @@ void JobsList::ActJobFinished(JobsList::iterator &i,
               cache_per_job_dirs.push_back(it->substr(0, it->find(" "))+"/joblinks");
             }
             job_clean_deleted(*i,config,cache_per_job_dirs);
-            i->job_state = JOB_STATE_DELETED;
+            SetJobState(i, JOB_STATE_DELETED, "Job stayed unattended too long");
             state_changed=true;
           } else {
             // delete everything
@@ -983,12 +990,16 @@ void JobsList::ActJobDeleted(JobsList::iterator &i,
         time_t t = -1;
         if(!job_local_read_cleanuptime(i->job_id,config,t) || ((time(NULL)-(t+i->keep_deleted)) >= 0)) {
           logger.msg(Arc::INFO,"%s: Job is ancient - delete rest of information",i->job_id);
+          UnlockDelegation(i); // not needed here but in case someting went wrong previously
           // delete everything
           job_clean_final(*i,config);
         }
 }
 
 bool JobsList::ActJob(JobsList::iterator &i) {
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), i->job_id);
+  job_state_t perflog_start_state = i->job_state;
+
   bool once_more     = true;
   bool delete_job    = false;
   bool job_error     = false;
@@ -1025,11 +1036,11 @@ bool JobsList::ActJob(JobsList::iterator &i) {
         }
         // special processing for INLRMS case
         if(i->job_state == JOB_STATE_INLRMS) {
-          i->job_state = JOB_STATE_CANCELING;
+          SetJobState(i, JOB_STATE_CANCELING, "Request to cancel job");
         }
         // if FINISHING we wait to get back all DTRs
         else if (i->job_state != JOB_STATE_PREPARING) {
-          i->job_state = JOB_STATE_FINISHING;
+          SetJobState(i, JOB_STATE_FINISHING, "Request to cancel job");
         }
         job_cancel_mark_remove(i->job_id,config);
         state_changed=true;
@@ -1084,14 +1095,14 @@ bool JobsList::ActJob(JobsList::iterator &i) {
             // so ignore them
           } else if(i->job_state == JOB_STATE_FINISHING) {
             // No matter if FINISHING fails - it still goes to FINISHED
-            i->job_state = JOB_STATE_FINISHED;
+            SetJobState(i, JOB_STATE_FINISHED, "Job processing error");
             if(GetLocalDescription(i)) {
               if (--(jobs_dn[i->local->DN]) <= 0) jobs_dn.erase(i->local->DN);
             }
             state_changed=true;
             once_more=true;
           } else {
-            i->job_state = JOB_STATE_FINISHING;
+            SetJobState(i, JOB_STATE_FINISHING, "Job processing error");
             state_changed=true;
             once_more=true;
           }
@@ -1130,22 +1141,22 @@ bool JobsList::ActJob(JobsList::iterator &i) {
               // analyze results
               if(result->action == ContinuationPlugins::act_fail) {
                 logger.msg(Arc::ERROR,"%s: Plugin at state %s : %s",
-                    i->job_id.c_str(),states_all[i->get_state()].name,
+                    i->job_id.c_str(),i->get_state_name(),
                     result->response);
                 i->AddFailure(std::string("Plugin at state ")+
-                states_all[i->get_state()].name+" failed: "+(result->response));
+                i->get_state_name()+" failed: "+(result->response));
                 job_error=true;
               } else if(result->action == ContinuationPlugins::act_log) {
                 // Scream but go ahead
                 logger.msg(Arc::WARNING,"%s: Plugin at state %s : %s",
-                    i->job_id.c_str(),states_all[i->get_state()].name,
+                    i->job_id.c_str(),i->get_state_name(),
                     result->response);
               } else if(result->action == ContinuationPlugins::act_pass) {
                 // Just continue quietly
               } else {
                 logger.msg(Arc::ERROR,"%s: Plugin execution failed",i->job_id);
                 i->AddFailure(std::string("Failed running plugin at state ")+
-                    states_all[i->get_state()].name);
+                    i->get_state_name());
                 job_error=true;
               }
               ++result;
@@ -1169,7 +1180,8 @@ bool JobsList::ActJob(JobsList::iterator &i) {
     } while(job_error);
     if(delete_job) { 
       logger.msg(Arc::ERROR,"%s: Delete request due to internal problems",i->job_id);
-      i->job_state = JOB_STATE_FINISHED; // move to finished in order to remove from list
+      // Move to finished in order to remove from list
+      SetJobState(i, JOB_STATE_FINISHED, "Preparing to delete job due to internal error");
       if(i->GetLocalDescription(config)) {
         if (--(jobs_dn[i->local->DN]) == 0) jobs_dn.erase(i->local->DN);
       }
@@ -1181,6 +1193,15 @@ bool JobsList::ActJob(JobsList::iterator &i) {
       once_more=true; // to process some things in local
     }
   }
+
+  if(perfrecord.Started()) {
+    job_state_t perflog_end_state = i->job_state;
+    std::string name(GMJob::get_state_name(perflog_start_state));
+    name += "-";
+    name += GMJob::get_state_name(perflog_end_state);
+    perfrecord.End(name);
+  };
+
   // FINISHED+DELETED jobs are not kept in list - only in files
   // if job managed to get here with state UNDEFINED -
   // means we are overloaded with jobs - do not keep them in list
@@ -1194,7 +1215,6 @@ bool JobsList::ActJob(JobsList::iterator &i) {
     } else {
       jobs_pending--;
     }
-    if(i->local) { delete i->local; }
     i=jobs.erase(i);
   }
   else {
@@ -1259,13 +1279,15 @@ bool JobsList::RestartJobs(const std::string& cdir,const std::string& odir) {
 bool JobsList::RestartJobs(void) {
   std::string cdir=config.control_dir;
   // Jobs from old version
-  bool res1 = RestartJobs(cdir,cdir+"/restarting");
+  bool res1 = RestartJobs(cdir,cdir+"/"+subdir_rew);
   // Jobs after service restart
-  bool res2 = RestartJobs(cdir+"/processing",cdir+"/restarting");
+  bool res2 = RestartJobs(cdir+"/"+subdir_cur,cdir+"/"+subdir_rew);
   return res1 && res2;
 }
 
 bool JobsList::ScanJobs(const std::string& cdir,std::list<JobFDesc>& ids) {
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
+
   try {
     Glib::Dir dir(cdir);
     for(;;) {
@@ -1292,10 +1314,14 @@ bool JobsList::ScanJobs(const std::string& cdir,std::list<JobFDesc>& ids) {
     logger.msg(Arc::ERROR,"Failed reading control directory: %s: %s",config.control_dir, e.what());
     return false;
   }
+
+  perfrecord.End("SCAN-JOBS");
   return true;
 }
 
 bool JobsList::ScanMarks(const std::string& cdir,const std::list<std::string>& suffices,std::list<JobFDesc>& ids) {
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
+
   try {
     Glib::Dir dir(cdir);
     for(;;) {
@@ -1329,37 +1355,47 @@ bool JobsList::ScanMarks(const std::string& cdir,const std::list<std::string>& s
     logger.msg(Arc::ERROR,"Failed reading control directory: %s",config.control_dir);
     return false;
   }
+
+  perfrecord.End("SCAN-MARKS");
   return true;
 }
 
 // find new jobs - sort by date to implement FIFO
 bool JobsList::ScanNewJobs(void) {
-  std::string cdir=config.control_dir;
-  std::list<JobFDesc> ids;
-  // For picking up jobs after service restart
-  std::string odir=cdir+"/restarting";
-  if(!ScanJobs(odir,ids)) return false;
-  // sorting by date
-  ids.sort();
-  for(std::list<JobFDesc>::iterator id=ids.begin();id!=ids.end();++id) {
-    iterator i;
-    AddJobNoCheck(id->id,i,id->uid,id->gid);
-  }
-  ids.clear();
-  // For new jobs
-  std::string ndir=cdir+"/accepting";
-  if(!ScanJobs(ndir,ids)) return false;
-  // sorting by date
-  ids.sort();
-  for(std::list<JobFDesc>::iterator id=ids.begin();id!=ids.end();++id) {
-    iterator i;
-    // adding job with file's uid/gid
-    AddJobNoCheck(id->id,i,id->uid,id->gid);
-  }
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
+  // New jobs will be accepted only if number of jobs being processed
+  // does not exceed allowed. So avoid scanning if no jobs will be allowed.
+  if((AcceptedJobs() < config.max_jobs) || (config.max_jobs == -1)) {
+    std::string cdir=config.control_dir;
+    std::list<JobFDesc> ids;
+    // For picking up jobs after service restart
+    std::string odir=cdir+"/"+subdir_rew;
+    if(!ScanJobs(odir,ids)) return false;
+    // sorting by date
+    ids.sort();
+    for(std::list<JobFDesc>::iterator id=ids.begin();id!=ids.end();++id) {
+      iterator i;
+      AddJobNoCheck(id->id,i,id->uid,id->gid);
+    };
+    ids.clear();
+    // For new jobs
+    std::string ndir=cdir+"/"+subdir_new;
+    if(!ScanJobs(ndir,ids)) return false;
+    // sorting by date
+    ids.sort();
+    for(std::list<JobFDesc>::iterator id=ids.begin();id!=ids.end();++id) {
+      iterator i;
+      // adding job with file's uid/gid
+      AddJobNoCheck(id->id,i,id->uid,id->gid);
+    };
+  };
+  perfrecord.End("SCAN-JOBS-NEW");
   return true;
 }
 
 bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
+
   // We are going to scan a dir with a lot of files here. So we scan it in
   // parts and limit scanning time. A finished job is added to the job list
   // and acted on straight away. If it remains finished or is deleted then it
@@ -1368,7 +1404,7 @@ bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
   // are normally processed in ScanNewMarks but can also happen here.
   time_t start = time(NULL);
   if(max_scan_time < 10) max_scan_time=10; // some sane number - 10s
-  std::string cdir=config.control_dir+"/finished";
+  std::string cdir=config.control_dir+"/"+subdir_old;
   try {
     if(!old_dir) {
       old_dir = new Glib::Dir(cdir);
@@ -1413,10 +1449,14 @@ bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
     }
     return false;
   }
+
+  perfrecord.End("SCAN-JOBS-OLD");
   return true;
 }
 
 bool JobsList::ScanNewMarks(void) {
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
+
   std::string cdir=config.control_dir;
   std::string ndir=cdir+"/"+subdir_new;
   std::list<JobFDesc> ids;
@@ -1445,16 +1485,20 @@ bool JobsList::ScanNewMarks(void) {
       i->job_state = st;
     }
   }
+
+  perfrecord.End("SCAN-MARKS-NEW");
   return true;
 }
 
 // For simply collecting all jobs. 
 bool JobsList::ScanAllJobs(void) {
+  Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
+
   std::list<std::string> subdirs;
-  subdirs.push_back("/restarting"); // For picking up jobs after service restart
-  subdirs.push_back("/accepting");  // For new jobs
-  subdirs.push_back("/processing"); // For active jobs
-  subdirs.push_back("/finished");   // For done jobs
+  subdirs.push_back(std::string("/")+subdir_rew); // For picking up jobs after service restart
+  subdirs.push_back(std::string("/")+subdir_new); // For new jobs
+  subdirs.push_back(std::string("/")+subdir_cur); // For active jobs
+  subdirs.push_back(std::string("/")+subdir_old); // For done jobs
   for(std::list<std::string>::iterator subdir = subdirs.begin();
                                subdir != subdirs.end();++subdir) {
     std::string cdir=config.control_dir;
@@ -1468,16 +1512,18 @@ bool JobsList::ScanAllJobs(void) {
       AddJobNoCheck(id->id,i,id->uid,id->gid);
     }
   }
+
+  perfrecord.End("SCAN-JOBS-ALL");
   return true;
 }
 
 bool JobsList::AddJob(const JobId& id) {
   if(FindJob(id) != jobs.end()) return true;
   std::list<std::string> subdirs;
-  subdirs.push_back("/restarting"); // For picking up jobs after service restart
-  subdirs.push_back("/accepting");  // For new jobs
-  subdirs.push_back("/processing"); // For active jobs
-  subdirs.push_back("/finished");   // For done jobs
+  subdirs.push_back(std::string("/")+subdir_rew); // For picking up jobs after service restart
+  subdirs.push_back(std::string("/")+subdir_new); // For new jobs
+  subdirs.push_back(std::string("/")+subdir_cur); // For active jobs
+  subdirs.push_back(std::string("/")+subdir_old); // For done jobs
   for(std::list<std::string>::iterator subdir = subdirs.begin();
                                subdir != subdirs.end();++subdir) {
     std::string cdir=config.control_dir;
