@@ -59,8 +59,8 @@ bool JobLog::start_info(GMJob &job,const GMConfig &config) {
     std::ofstream o;
     if(!open_stream(o)) return false;
     o<<"Started - job id: "<<job.get_id()<<", unix user: "<<job.get_user().get_uid()<<":"<<job.get_user().get_gid()<<", ";
-    if(job.GetLocalDescription(config)) {
-      JobLocalDescription *job_desc = job.get_local();
+    JobLocalDescription *job_desc = job.GetLocalDescription(config);
+    if(job_desc) {
       std::string tmps;
       tmps=job_desc->jobname;
       tmps = Arc::escape_chars(tmps, "\"\\", '\\', false);
@@ -81,8 +81,8 @@ bool JobLog::finish_info(GMJob &job,const GMConfig &config) {
     if(!open_stream(o)) return false;
     o<<"Finished - job id: "<<job.get_id()<<", unix user: "<<job.get_user().get_uid()<<":"<<job.get_user().get_gid()<<", ";
     std::string tmps;
-    if(job.GetLocalDescription(config)) {
-      JobLocalDescription *job_desc = job.get_local();
+    JobLocalDescription *job_desc = job.GetLocalDescription(config);
+    if(job_desc) {
       tmps=job_desc->jobname;
       tmps = Arc::escape_chars(tmps, "\"\\", '\\', false);
       o<<"name: \""<<tmps<<"\", ";
@@ -127,7 +127,12 @@ bool JobLog::RunReporter(const GMConfig &config) {
     logger.msg(Arc::ERROR,": Failure creating slot for reporter child process");
     return false;
   };
-  proc->AssignInitializer(&initializer,(void*)&config);
+  std::string errlog = config.ControlDir() + "/job.logger.errors"; // backward compatibility
+  JobLog* joblog = config.GetJobLog();
+  if(joblog) {
+    if(!joblog->logfile.empty()) errlog = joblog->logfile;
+  };
+  proc->AssignInitializer(&initializer,(void*)errlog.c_str());
   logger.msg(Arc::DEBUG, "Running command %s", cmd);
   if(!proc->Start()) {
     delete proc;
@@ -170,7 +175,7 @@ bool JobLog::make_file(GMJob &job, const GMConfig& config) {
   JobLocalDescription* local;
   if(!job.GetLocalDescription(config)) {
     result=false;
-  } else if((local=job.get_local()) == NULL) { 
+  } else if((local=job.GetLocalDescription(config)) == NULL) { 
     result=false;
   } else {
     if(!(local->jobreport.empty())) {
@@ -202,27 +207,14 @@ JobLog::~JobLog(void) {
 }
 
 void JobLog::initializer(void* arg) {
-  GMConfig& config = *(GMConfig*)arg;
-  JobLog* joblog = config.GetJobLog();
-  // set good umask
-  ::umask(0077);
-  // close all handles inherited from parent
-  struct rlimit lim;
-  rlim_t max_files = RLIM_INFINITY;
-  if(::getrlimit(RLIMIT_NOFILE,&lim) == 0) { max_files=lim.rlim_cur; };
-  if(max_files == RLIM_INFINITY) max_files=4096; // sane number
-  for(int i=0;i<max_files;i++) { ::close(i); };
+  char const * errlog = (char const *)arg;
   int h;
   // set up stdin,stdout and stderr
   h=::open("/dev/null",O_RDONLY);
   if(h != 0) { if(dup2(h,0) != 0) { sleep(10); exit(1); }; close(h); };
   h=::open("/dev/null",O_WRONLY);
   if(h != 1) { if(dup2(h,1) != 1) { sleep(10); exit(1); }; close(h); };
-  std::string errlog = config.ControlDir() + "/job.logger.errors"; // backward compatibility
-  if(joblog) {
-    if(!joblog->logfile.empty()) errlog = joblog->logfile;
-  };
-  h=::open(errlog.c_str(),O_WRONLY | O_CREAT | O_APPEND,S_IRUSR | S_IWUSR);
+  h=errlog ? ::open(errlog,O_WRONLY | O_CREAT | O_APPEND,S_IRUSR | S_IWUSR) : -1;
   if(h==-1) { h=::open("/dev/null",O_WRONLY); };
   if(h != 2) { if(dup2(h,2) != 2) { sleep(10); exit(1); }; close(h); };
 }
