@@ -114,6 +114,8 @@ my $gmcommon_options = {
     scratchdir => '*',
     enable_emies_interface => '*',
     enable_arc_interface => '*',
+    enable_perflog_reporting => '*',
+    perflogdir => '*'
 };
 my $ldap_infosys_options = {
     SlapdPort => '*',
@@ -234,7 +236,7 @@ my $allbools = [ qw(
                  PublishNordugrid Homogeneous VirtualMachine
                  ConnectivityIn ConnectivityOut Preemption
                  infosys_nordugrid infosys_glue12 infosys_glue2_ldap infosys_glue2_ldap_showactivities
-                 GridftpdEnabled GridftpdAllowNew Distributed enable_arc_interface enable_emies_interface ) ];
+                 GridftpdEnabled GridftpdAllowNew Distributed enable_arc_interface enable_emies_interface enable_perflog_reporting) ];
 
 ############################ Generic functions ###########################
 
@@ -968,6 +970,20 @@ sub printLRMSConfigScript {
     _print_shell_end();
 }
 
+## Cherry picks arc.conf values
+## this does not work properly now since arc.conf is parsed
+## into the strange GLUE2-like intermediate representation
+## described at the top of this file.
+## input: configfile,configblock, configoption
+sub getValueOf ($$$){
+   my ($configfile,$block,$option) = @_;
+   my $config = ConfigCentral::parseConfig($configfile);	
+   if ($block eq 'common') {
+	   return $config->{$option};
+   } else {
+      return $config->{$block}{$option};
+   }
+}
 
 1;
 
