@@ -119,7 +119,7 @@ namespace DataStaging {
         LogToRootLogger(Arc::WARNING, "Failed to remove temporary proxy "+proxy_file+": "+Arc::StrError(errno));
       }
     }
-    --current_processes;
+    if (current_processes > 0) --current_processes;
   }
 
   /*
@@ -347,6 +347,7 @@ namespace DataStaging {
          <ErrorLocation>1</ErrorLocation>
          <Log>...</Log>
          <BytesTransferred>1234</BytesTransferred>
+         <TransferTime>123456789</TransferTime>
          <Checksum>adler32:a123a45</Checksum>
        </Result>
        ...
@@ -404,6 +405,7 @@ namespace DataStaging {
         resultelement.NewChild("ErrorDescription") = dtr->get_error_status().GetDesc();
         resultelement.NewChild("ErrorStatus") = Arc::tostring(dtr->get_error_status().GetErrorStatus());
         resultelement.NewChild("ErrorLocation") = Arc::tostring(dtr->get_error_status().GetErrorLocation());
+        resultelement.NewChild("TransferTime") = Arc::tostring(dtr->get_transfer_time());
         archived_dtrs_lock.lock();
         archived_dtrs[dtrid] = std::pair<std::string, std::string>("TRANSFER_ERROR", dtr->get_error_status().GetDesc());
         archived_dtrs_lock.unlock();
@@ -411,6 +413,7 @@ namespace DataStaging {
       else if (dtr->get_status() == DTRStatus::TRANSFERRED) {
         logger.msg(Arc::INFO, "DTR %s finished successfully", dtrid);
         resultelement.NewChild("ResultCode") = "TRANSFERRED";
+        resultelement.NewChild("TransferTime") = Arc::tostring(dtr->get_transfer_time());
         // pass calculated checksum back to Scheduler (eg to insert in catalog)
         if (dtr->get_destination()->CheckCheckSum()) resultelement.NewChild("CheckSum") = dtr->get_destination()->GetCheckSum();
         archived_dtrs_lock.lock();
@@ -523,6 +526,7 @@ namespace DataStaging {
          <ReturnCode>ERROR</ReturnCode>
          <ErrorDescription>...</ErrorDescription>
          <AllowedDir>/var/arc/cache</AllowedDir>
+         <LoadAvg>6.5</LoadAvg>
          ...
        </Result>
        ...
@@ -538,6 +542,15 @@ namespace DataStaging {
       resultelement.NewChild("AllowedDir") = *dir;
     }
 
+    // Send the 5 min load average
+    double avg[3];
+    if (getloadavg(avg, 3) != 3) {
+      logger.msg(Arc::WARNING, "Failed to get load average: %s", Arc::StrError());
+      resultelement.NewChild("LoadAvg") = "-1";
+    } else {
+      resultelement.NewChild("LoadAvg") = Arc::tostring(avg[1]);
+    }
+
     return Arc::MCC_Status(Arc::STATUS_OK);
   }
 
@@ -721,7 +734,7 @@ namespace DataStaging {
 
 } // namespace DataStaging
 
-Arc::PluginDescriptor ARC_PLUGINS_TABLE_NAME[] = {
+extern Arc::PluginDescriptor const ARC_PLUGINS_TABLE_NAME[] = {
     { "datadeliveryservice", "HED:SERVICE", NULL, 0, &DataStaging::get_service },
     { NULL, NULL, NULL, 0, NULL }
 };
