@@ -18,12 +18,12 @@
 
 static Arc::Logger logger(Arc::Logger::getRootLogger(),"AuthUserVOMS");
 
-static int process_vomsproxy(const char* filename,std::vector<struct voms> &data,bool auto_cert = false);
+static AuthResult process_vomsproxy(const char* filename,std::vector<struct voms_t> &data,bool auto_cert = false);
 
-int AuthUser::process_voms(void) {
+AuthResult AuthUser::process_voms(void) {
   if(!voms_extracted) {
     if(filename.length() > 0) {
-      int err = process_vomsproxy(filename.c_str(),voms_data);
+      AuthResult err = process_vomsproxy(filename.c_str(),voms_data);
       voms_extracted=true;
       logger.msg(Arc::DEBUG, "VOMS proxy processing returns: %i - %s", err, err_to_string(err));
       if(err != AAA_POSITIVE_MATCH) return err;
@@ -32,7 +32,7 @@ int AuthUser::process_voms(void) {
   return AAA_POSITIVE_MATCH;
 }
 
-int AuthUser::match_voms(const char* line) {
+AuthResult AuthUser::match_voms(const char* line) {
   // parse line
   std::string vo("");
   std::string group("");
@@ -73,25 +73,26 @@ int AuthUser::match_voms(const char* line) {
   if(process_voms() != AAA_POSITIVE_MATCH) return AAA_FAILURE;
   if(voms_data.empty()) return AAA_NO_MATCH;
   // analyse permissions
-  for(std::vector<struct voms>::iterator v = voms_data.begin();v!=voms_data.end();++v) {
+  for(std::vector<struct voms_t>::iterator v = voms_data.begin();v!=voms_data.end();++v) {
     logger.msg(Arc::DEBUG, "Match vo: %s", v->voname);
     if((vo == "*") || (vo == v->voname)) {
-      for(std::vector<struct voms_attrs>::iterator d=v->attrs.begin();d!=v->attrs.end();++d) {
-        logger.msg(Arc::VERBOSE, "Match group: %s", d->group);
-        logger.msg(Arc::VERBOSE, "Match role: %s", d->role);
-        logger.msg(Arc::VERBOSE, "Match capabilities: %s", d->cap);
-        if(((group == "*") || (group == d->group)) &&
-           ((role == "*") || (role == d->role)) &&
-           ((capabilities == "*") || (capabilities == d->cap))) {
-          logger.msg(Arc::VERBOSE, "Match: %s %s %s %s",v->voname,d->group,d->role,d->cap);
-          default_voms_=v->server.c_str();
-          default_vo_=v->voname.c_str();
-          default_role_=d->role.c_str();
-          default_capability_=d->cap.c_str();
-          default_vgroup_=d->group.c_str();
-          return AAA_POSITIVE_MATCH;
+      bool matched = false;
+      for(std::vector<struct voms_fqan_t>::iterator f = v->fqans.begin(); f != v->fqans.end(); ++f) {
+        if(((group == "*") || (group == f->group)) &&
+           ((role == "*") || (role == f->role)) &&
+           ((capabilities == "*") || (capabilities == f->capability))) {
+          if(!matched) {
+            default_voms_ = voms_t();
+            default_voms_.voname = v->voname;
+            default_voms_.server = v->server;
+            matched = true;
+          };
+          default_voms_.fqans.push_back(*f);
         };
       };
+      if(matched) {
+        return AAA_POSITIVE_MATCH;
+      };
     };
   };
   logger.msg(Arc::VERBOSE, "Matched nothing");
@@ -99,7 +100,7 @@ int AuthUser::match_voms(const char* line) {
 }
 
 
-static int process_vomsproxy(const char* filename,std::vector<struct voms> &data,bool /* auto_cert */) {
+static AuthResult process_vomsproxy(const char* filename,std::vector<struct voms_t> &data,bool /* auto_cert */) {
   std::string voms_dir = "/etc/grid-security/vomsdir";
   std::string cert_dir = "/etc/grid-security/certificates";
   {
